/*****************************************************************************
    This file should be included into your application modules which call
    the chart interface functions.
*****************************************************************************/
#if defined (_WIN32)
   #if !defined(WIN32)
     #define WIN32
   #endif
#endif
#if defined (WIN32)
   #define huge
   #define _export
#else
   #define huge huge 
#endif
#define CHART32

/*****************************************************************************
   Global Constants
*****************************************************************************/
#define MAX_LINE_POINTS 50
#define MAX_HLBS        200
#define MAX_POINTS      2000

#define UNIT_PHYSICAL  0
#define UNIT_LOGICAL   1

#define BAR_HORZ       0
#define BAR_VERT       1
#define BAR_VERT_LABEL_HORZ 2

#define HLB_HORZ       0
#define HLB_VERT       1

#define LINE_HORZ      0
#define LINE_VERT      1

#define POS_ABS        0
#define POS_NEXT       1
#define POS_STACK      2
#define POS_BEHIND     3

#define POS_CENTER     0
#define POS_OUT        1
#define POS_UP         2
#define POS_DOWN       3

#define SHAPE_RECT     0
#define SHAPE_CYND     1

#define VPOS_TOP       0
#define VPOS_BOT       1
#define VPOS_CENTER    2

#define VPOS_PARAN     3

#define LEG_HIDE       0
#define LEG_TOP        1
#define LEG_LEFT       2
#define LEG_RIGHT      3
#define LEG_BOT        4
#define LEG_TOP_LEFT   5
#define LEG_TOP_RIGHT  6
#define LEG_BOT_LEFT   7
#define LEG_BOT_RIGHT  8
#define LEG_SPECIFY    9

#define SYM_NONE       0
#define SYM_CIRCLE     1
#define SYM_RECT       2
#define SYM_DIAMOND    3
#define SYM_TRI        4
#define SYM_CHAR       5
#define SYM_LINE       6

#define DESC_TOP       0
#define DESC_BOT       1

#define SHOW_LOW       0x1
#define SHOW_HIGH      0x2
#define SHOW_OPEN      0x4
#define SHOW_CLOSE     0x8

#define MAX_CHARTS  50
#define MAX_LABELS 100

#define CHART_BAR  0
#define CHART_PIE  1
#define CHART_LIN  2
#define CHART_HLB  3
#define CHART_PNT  4

#define AXIS_X     0
#define AXIS_Y     1
#define AXIS_Z     2
#define AXIS_Y1    3

#define TITLE_MAIN 0
#define TITLE_SUB  1

#define MARK_MAJOR 0
#define MARK_MINOR 1

#define SURFACE_FRONT 0
#define SURFACE_SIDE  1

#define TAB_OPEN   0
#define TAB_CLOSE  1

#define DATA_HEADER 0
#define DATA_ITEM   1
#define DATA_LEGEND 2
#define DATA_LABEL  3

#define  AXIS_MAX    0
#define  AXIS_MIN    1
#define  AXIS_ORIG   2
#define  AXIS_MAJOR  3
#define  AXIS_MINOR  4

#define  SOBJ_POLYGON 0
#define  SOBJ_ELLIPSE 1
#define  SOBJ_PIE     2
#define  SOBJ_RECT    3
#define  SOBJ_LINE    4

#define  LINE_NONE     0
#define  LINE_STRAIGHT 1
#define  LINE_SMOOTH   2

#define  RES_META_DC   720

             //********** Error Codes ********
#define ERR_BAR       1
#define ERR_BKND      2
#define ERR_AXIS      3
#define ERR_LEGEND    4
#define ERR_TITLE     5
#define ERR_PIE       6
#define ERR_LINE      7
#define ERR_HLB       8
#define ERR_POINT     9
#define ERR_JPEG     10
#define ERR_IMAGE    11

#define true  TRUE
#define false FALSE
#define null  NULL
#define True(x)                     (x)
#define False(x)                    (!(x))
#define bool                        BOOL

/****************************************************************************
   Instructions for packing the following structure
*****************************************************************************/
#ifndef RC_INVOKED
   #pragma pack(1)
#endif

//***************************************************************************
                      
               //**  BAR CHART PARAMETER STRCTURES **
               //====================================

/*****************************************************************************
   The parameter structure definition that is used by your application
   to define x or y axes of a bar chart.
*****************************************************************************/
struct StrBarAxis {
       BOOL     draw;           // TRUE to draw the axis 

       int      orig;           /* Origin value for the axis in your
                                   application unit */
       int      max;            /* Rightmost value for the X axis or
                                   the topmost value for the Y axis.  
                                   This value is specified in your application
                                   unit.
                                   This value must be greater than 'min'.

                                   If you do not wish to draw the higher
                                   side of the axis, specify max=orig. */

       int      min;            /* Leftmost value for the X axis or
                                   the bottommost value for the Y axis.  
                                   This value is specified in your application
                                   unit.
                                   This value must be less than 'max'.
                                   
                                   If you do not wish to draw the lower 
                                   side of the axis, specify min=orig. */
                                    
       COLORREF color;          // color of the  axis line 
       int      depth;          // axis plane depth in mm, use -1 for default 
       
       int      Interval1;      /* Major axis mark interval in application 
                                   unit */
       int      Interval2;      /* Minor axis mark interval in application 
                                   unit */
       int      Len1;           // length of major axis marks in mm 
       int      Len2;           // length of minor axis marks in mm 
       COLORREF Color1,         // Color to draw the major marks 
                Color2;         // Color to draw the minor marks 
       BOOL     PrintMajorValue;// print the value for the major marks 

       char     title[50];      // title for the axis 
       int      TitleX;         /* X position (mm) of the axis title relative 
                                   to the left chart position.  Specify -1 to 
                                   use the default. The default value centers 
                                   the title of the X axis with bottom margin,
                                   or centers the title of the Y axis 
                                   within left margin */
       int      TitleY;         /* Y position (mm) of the title relative to the 
                                   top chart position.  Specify -1 to use
                                   the default.  The default value centers 
                                   the title of the X axis within bottom margin,
                                   or centers the title of the Y axis
                                   within left margin */
       COLORREF TitleColor;     // color of the axis title 

       HFONT    hFont;          /* Font used to write the axis title, specify 
                                   0 to use default font.*/

       };    

/*****************************************************************************
   This parameter structure defines the bar chart title fields.  
*****************************************************************************/
struct StrBarTitle {
       char     text[60];       /* Chart title. If the text length is
                                   zero, the corresponding title is
                                   not printed.*/
       int      x;              /* X position (mm) of the title relative to the 
                                   top left chart position.  Specify -1 to 
                                   center horizontally */
       int      y;              /* Y position (mm) of the title relative to the 
                                   top left chart position.  Specify -1 to use
                                   the default.  The default value positions
                                   the main title centered within the
                                   top margin, and positions the subtitle
                                   below the main title. */

       COLORREF TextColor;      // Foreground color for the title 
       BOOL     opaque;         /* Print title with opaque background, specify
                                   the background color using the BackColor 
                                   field */ 
       COLORREF BackColor;      /* Background color for the title, needed only
                                   when 'opaque' is TRUE */
       HFONT    hFont;          /* Font used to write the title, specify 0
                                   to use the default font. The default font
                                   for the main title is 18 point size.
                                   The default font for the
                                   subtitle is 14 point size. */
       };

/*****************************************************************************
   The parameter structure definition that is used by your application
   to specify the general bar chart parameters.
   Use the 'InitBarHdr' function to initialize this structure before 
   assigning your application values.
*****************************************************************************/

 struct StrBarHdr { 
       HDC      hDC;            /* Display context of the output window or 
                                   printer */

       HANDLE   hInst;          // Handle of the current instanaces. 

       BOOL     show;           // Set to TRUE to show the picture on the device 
       BOOL     ReturnBitmap;   /* Set to TRUE to return the handle of the 
                                   chart bitmap compatible with the display
                                   device */
       HBITMAP  hBM;            /* (output) The DLL returns the chart bitmap
                                   handle when ReturnBitmap is TRUE */
       
       int      BarShape;       /* SHAPE_RECT = rectangular bars, 
                                   SHAPE_CYND = cylindrical bars */

               //** Chart Position/Size Parameters **
       int      UnitType;       /* ChartPro uses millimeters (mm) to specify 
                                   many chart dimension parameters.  You can
                                   set the millimeter unit to logical (UNIT_LOGICAL)
                                   or physical (UNIT_PHYSICAL). The default
                                   is UNIT_PHYSICAL */

       int      x;              /* Top-left X position of the bar chart in 
                                   device units */
       int      y;              /* Top-left Y position of the bar char in 
                                   device units */
       int      width;          // Width of the bar chart in mm (millimeters) 
       int      height;         // Height of the bar chart in mm (millimeters) 

       int      PosFlag;        /* BAR_HORZ=Position the bars on the X axis
                                   BAR_VERT=Position the bars on the Y axis 
                                   BAR_VERT_LABEL_HORZ=Position the bar on Y axis with axis labels printed horizontally */

       int      HorzAngle;      /* Horizontal view angle (-60 to 60). The zero
                                   angle hides the left and right sides of
                                   the bars */
       int      VertAngle;      /* Vertical view angle (-60 to 60). The zero
                                   angle hides the top and bottom sides of
                                   the bars */

                      //** Chart Margins **
       int      LeftMargin,     // Left Margin in mm (millimeter) 
                RightMargin,    // Right Margin in mm 
                TopMargin,      // Top Margin in mm 
                BotMargin;      // Bottom margin in mm 

               //** Chart background Parameters **
       BOOL     ClearBackground; /* TRUE=To clear background, FALSE=leave the 
                                   background as it is. The following two 
                                   values must be specified when ClearBackground
                                   is TRUE*/
       COLORREF ChartBackground,// Background color for the entire chart 
                BarBackground;  // Background colors behind the bars 

       int      LabelSize;      /* Point size of the text to label the axis
                                   and bar values (1 point = 1/72 inches ) */
       char     LabelTypeface[32]; // font type face for labelling 

               //********** Chart Titles ********
       struct  StrBarTitle main,// main title information 
                           sub; // subtitle information 

               //********** Chart Axes ********
       struct  StrBarAxis ax,   // information about the x axis 
                          ay,   // information about the y axis 
                          ay1;  // information about the second y axis 
       
       BOOL (CALLBACK _export *AxisValueLabel)(char,int,LPSTR); /* Process instance
                                   of the routine to provide axis value labels.
                                   You can specify NULL for this parameter
                                   if you wish the DLL to create axis value
                                   labels automatically */  
                       
               //** Bar parameters **
       int      BarCount;       // number of bars to display 
       struct   StrBar far *bar;/* pointer to a structure containing 
                                   parameters for each bar.*/

               //** Legend parameters **
       int      LegLoc;         /* Legend box location: 
                                   LEG_HIDE, LEG_TOP, LEG_BOT, LEG_LEFT,
                                   LEG_RIGHT, LEG_TOP_LEFT, LEG_TOP_RITGHT,
                                   LEG_BOT_LEFT, LEG_BOT_RIGHT,
                                   and LEG_SPECIFY */

       int      LegX,           /* x position (mm) of the legend box relative
                                   to the chart box (when LegLoc==LEG_SPECIFY) */

                LegY;           /* y position (mm) of the legend box with
                                   respect to the chart box (when LegLoc ==
                                   LEG_SPECIFY) */

       COLORREF LegColor;       // legend box color 

       int      LegCount;       // number of legends to display. 

       struct   StrBarLegend far *legend;/* pointer to a structure containing 
                                   each legend description.*/

                // Subobject location information OUTPUT by the DLL;
       struct   StrSubObj far *subobj;  /* this pointer should be initialized
                                           to NULL before calling for the first
                                           time.  The DLL returns the x/y locations
                                           of the chart subobjects in the memory block
                                           pointed by this pointer.  On subsequent
                                           calls, this pointer should be left
                                           at the value returned by the DLL */
       int      SubobjCount;            /* number of subobjects in the location
                                           pointed by the subobj pointer variable */
       };

/*****************************************************************************
   The parameter structure definition that is used by your application
   to specify the individual bar parameters.
   Use the 'InitBar' function to initialize this structure for every bar before 
   assigning your application values.
*****************************************************************************/
 struct StrBar { 
       int      flag;           /* Position Flags.  This flag specifies how
                                   'pos', 'PosZ' fields are interpreted.
                                   POS_ABS   = Absolute Positioning.
                                               Positions according to the your
                                               application value in the
                                               'pos' field.  The 'PosZ' field
                                               specifies the 'Z' axis position. 
                                   POS_NEXT  = Position next to the previous
                                               bar. 
                                   POS_STACK = Position on top of the 
                                               previous bar.
                                   POS_BEHIND= Position behind the previous
                                               bar. The view angle (see 
                                               StrBarHdr) must be non-zero 
                                               to see this bar */
       int      pos;            /* specifies the position of the bar
                                   when the 'flag' field is equal to 
                                   POS_ABS.  The position is specified using
                                   your application unit. 
                                    
                                   If PosFlag (see StrBarHdr) is equal to
                                   BAR_HORZ, the 'pos' must be within
                                   'max' and 'min' values for the X axis.
                                   When PosFlag is equal to BAR_VERT, the
                                   'pos' must be within 'max' and 'min' values
                                   for the Y axis.*/

       int      value;          /* A value that this bar represents, specify in
                                   your application unit. When PosFlag 
                                   (see StrBarHdr) is equal to BAR_HORZ, 
                                   the 'value' must be within 'max' and 'min'
                                   values for the Y axis. Similarly, when 
                                   PosFlag is equal to BAR_VERT, the 'value' 
                                   must be within 'max' and 'min' values for
                                   the X axis */

       BOOL     UseY1;          // TRUE if Y values are relative to the second Y axis

       int      PosZ;           // 'Z' position in mm 

       LOGBRUSH FrontBrush,     // brush to paint the front side of the bar 
                SideBrush;      // brush to paint other sides.

       COLORREF OutlineColor;   // color of the outlines of the bar 

       BOOL     solid;          /* TRUE to draw solid bars, otherwise only
                                   the outlines are drawn */

       BOOL     shadow;         /* TRUE to draw shadow.  Shadowed bars don't
                                   have sides */

       int      width;          /* width of the bar. A positive number 
                                   specifies the width in millimeters.  
                                   A negative number specified width in the 
                                   application units.  The absolute of the 
                                   negative number is used. 
                                   
                                   When drawing cyliderical bars, this
                                   field specifies the diameter of the bars.*/

       int      depth;          /* dept of the bar in mm (millimeters)  
                                   The view angles (see StrBarHdr) must be 
                                   non-zero to actually see the bar depth.
                                   
                                   This field is not used when drawing
                                   the cyliderical bars. */

       char     desc[30];       /* Bar description.  To omit the bar 
                                   description, specify a zero length 
                                   descriptoin.

                                   Bar description should be used sparingly
                                   to avoid crowded appearance */

       COLORREF DescColor;      // color used to print the description 
       BOOL     DescBox;        /* Show the description in a description box,
                                   otherwise the description appears under
                                   the bar */
       COLORREF DescBoxColor;   // color of the description box                              

       BOOL     PrintValue;     /* When TRUE, the value of the bar is printed.
                                   Also specify the following fields */
       int      ValuePos;       /* VPOS_TOP=Print the Value at the top
                                   of the bar, VPOS_BOT=Print the value 
                                   at the bottom of the bar*/
       char     ValueDesc[10];  /* specify a value string to print.  If a 
                                   zero length string is specified, a 
                                   string is constructed using the 'value'
                                   field */

       };

/*****************************************************************************
   The parameter structure defines each legend used by the bar charts.
   Use the 'InitBarLegend' function to initialize this structure for 
   every legend before assigning your application values.
*****************************************************************************/
 struct StrBarLegend { 
       char     desc[30];       // Legend description.  
       
       COLORREF color;          /* outline color of the legend symbol, and
                                   the color of the text */

       LOGBRUSH  brush;         // brush to fill the legend symbol 

      };

//***************************************************************************

               //**  PIE CHART PARAMETER STRCTURES **
               //====================================

/*****************************************************************************
   This parameter structure defines the pie chart title fields.  
*****************************************************************************/
 struct StrPieTitle {
       char     text[60];       /* Chart title. If the text length is
                                   zero, the corresponding title is
                                   not printed.*/
       int      x;              /* X position (mm) of the title relative to the 
                                   top left chart position.  Specify -1 to 
                                   center horizontally */
       int      y;              /* Y position (mm) of the title relative to the 
                                   top left chart position.  Specify -1 to use
                                   the default.  The default value positions
                                   the main title centered within the
                                   top margin, and positions the subtitle
                                   below the main title. */

       COLORREF TextColor;      // Foreground color for the title 
       BOOL     opaque;         /* Print title with opaque background, specify
                                   the background color using the BackColor 
                                   field */ 
       COLORREF BackColor;      /* Background color for the title, needed only
                                   when 'opaque' is TRUE */
       HFONT    hFont;          /* Font used to write the title, specify 0
                                   to use the default font. The default font
                                   for the main title is 18 point size.  
                                   The default font for the subtitle is 
                                   is 14 point size. */
       };

/*****************************************************************************
   The parameter structure definition that is used by your application
   to specify the general pie chart parameters.
   Use the 'InitPieHdr' function to initialize this structure before 
   assigning your application values.
*****************************************************************************/

 struct StrPieHdr { 
       HDC      hDC;            /* Display context of the output window or 
                                   printer */

       HANDLE   hInst;          // Handle of the current instanaces. 

       BOOL     show;           // Set to TRUE to show the picture on the device 
       BOOL     ReturnBitmap;   /* Set to TRUE to return the handle of the 
                                   chart bitmap compatible with the display
                                   device */
       HBITMAP  hBM;            /* (output) The DLL returns the chart bitmap
                                   handle when ReturnBitmap is TRUE */
       
               //** Chart Position/Size Parameters **
       int      UnitType;       /* ChartPro uses millimeters (mm) to specify 
                                   many chart dimension parameters.  You can
                                   set the millimeter unit to logical (UNIT_LOGICAL)
                                   or physical (UNIT_PHYSICAL). The default
                                   is UNIT_PHYSICAL */

       int      x;              /* Top-left X position of the bar chart in 
                                   device units */
       int      y;              /* Top-left Y position of the bar char in 
                                   device units */
       int      width;          // Width of the bar chart in mm (millimeters) 
       int      height;         // Height of the bar chart in mm (millimeters) 

       int      VertAngle;      // Vertical view angle (0 to 60). 

       int      InitAngle;      /* angle to place the first pie (0 to 360).  
                                   The 0 angle begins the first pie at the 
                                   right x axis.  The successive pies are 
                                   placed counter clockwise. */

                      //** Chart Margins **
       int      LeftMargin,     // Left Margin in mm (millimeter) 
                RightMargin,    // Right Margin in mm 
                TopMargin,      // Top Margin in mm 
                BotMargin;      // Bottom margin in mm 

               //** Chart background Parameters **
       BOOL     ClearBackground; /* TRUE=To clear background, FALSE=leave the 
                                   background as it is. The following two 
                                   values must be specified when ClearBackground
                                   is TRUE*/
       COLORREF ChartBackground;// Background color for the entire chart 

       int      LabelSize;      /* Point size of the text to label the axis
                                   and bar values (1 point = 1/72 inches ) */
       char     LabelTypeface[32]; // font type face for labelling 

               //********** Chart Titles ********
       struct  StrPieTitle main,// main title information 
                           sub; // subtitle information 

               //** Pie parameters **
       int      PieCount;       // number of pies pieces 
       struct   StrPie far *pie;/* pointer to a structure containing 
                                   parameters for each pie piece.*/

               //** Legend parameters **
       int      LegLoc;         /* Legend box location: 
                                   LEG_HIDE, LEG_TOP, LEG_BOT, LEG_LEFT,
                                   LEG_RIGHT, LEG_TOP_LEFT, LEG_TOP_RITGHT,
                                   LEG_BOT_LEFT, LEG_BOT_RIGHT,
                                   and LEG_SPECIFY */

       int      LegX,           /* x position (mm) of the legend box relative
                                   to the chart box (when LegLoc==LEG_SPECIFY) */

                LegY;           /* y position (mm) of the legend box with
                                   respect to the chart box (when LegLoc ==
                                   LEG_SPECIFY) */

       COLORREF LegColor;       // legend box color 

       int      LegCount;       // number of legends to display. 

       struct   StrPieLegend far *legend;/* pointer to a structure containing 
                                   each legend description.*/

                // Subobject location information OUTPUT by the DLL;
       struct   StrSubObj far *subobj;  /* this pointer should be initialized
                                           to NULL before calling for the first
                                           time.  The DLL returns the x/y locations
                                           of the chart subobjects in the memory block
                                           pointed by this pointer.  On subsequent
                                           calls, this pointer should be left
                                           at the value returned by the DLL */
       int      SubobjCount;            /* number of subobjects in the location
                                           pointed by the subobj pointer variable */
       };

/*****************************************************************************
   The parameter structure definition that is used by your application
   to specify the individual pie parameters.
   Use the 'InitPie' function to initialize this structure for every pie before 
   assigning your application values.
*****************************************************************************/
 struct StrPie {
       BOOL     show;           /* When TRUE, the pie piece is displayed.  You
                                   must provide the remaining parameters in 
                                   this structure even when the 'show' variable
                                   is FALSE. */

       int      flag;           /* Position Flags.  This flag specifies how
                                   'pos' field is interpreted.
                                   POS_CENTER= position at the center of 
                                               the pie.  The 'pos' field
                                               is ignored.
                                   POS_OUT   = The pie is pulled outward from
                                               the center.  The 'pos' field 
                                               indicates number of millimeters 
                                               that the pie is pulled outward.

                                   POS_UP    = The pid is move upward from
                                               the center position. The 'pos'
                                               field indicates the number
                                               of millimeters that the pie
                                               is moved upward.  The view
                                               angle (see StrPieHdr) must be 
                                               non-zero to see this kind of 
                                               displacement.

                                   POS_DOWN  = The pid is move downward from
                                               the center position. The 'pos'
                                               field indicates the number
                                               of millimeters that the pie
                                               is moved downward.  The view
                                               angle must be non-zero to
                                               see this kind of displacement.
                                   */

       int      pos;            /* Number of millimeters a pie is displaced
                                   from the center position. The direction
                                   of the displacement is given by the 
                                   'flag' field. */

       int      value;          /* A value that this pie represents, specify in
                                   your application unit. */

       LOGBRUSH FrontBrush,    // brush to paint the front side of the pie 
                SideBrush;     // brush to paint the height of the pie.

       COLORREF OutlineColor;   // color of the outlines of the pie 

       BOOL     solid;          /* TRUE to draw solid bars, otherwise only
                                   the outlines are drawn */

       int      depth;          /* dept of the pie in mm (millimeters)  
                                   The view angle (see StrPieHdr) must be 
                                   non-zero to actually see the pie depth. */

       char     desc[30];       /* Pie description.  To omit the pie 
                                   description, specify a zero length 
                                   description. */

       COLORREF DescColor;      // color used to print the description 
       BOOL     DescBox;        /* Show the description in a description box,
                                   otherwise the description appears next to
                                   the pie */
       COLORREF DescBoxColor;   // color of the description box                              

       BOOL     PrintValue;     /* When TRUE, the value of the pie is printed.
                                   Also specify the following fields */
       int      ValuePos;       /* VPOS_TOP=Print the Value over the
                                   description, VPOS_BOT=Print the value 
                                   under the description, VPOS_PARAN= Print
                                   the value within paranthesis beside the
                                   description */
       char     ValueDesc[10];  /* specify a value string to print.  If a 
                                   zero length string is specified, a 
                                   string is constructed using the 'value'
                                   field */

       };

/*****************************************************************************
   The parameter structure defines each legend.
   Use the 'InitPieLegend' function to initialize this structure for 
   every legend before assigning your application values.
*****************************************************************************/
 struct StrPieLegend { 
       char     desc[30];       // Legend description.  
       
       COLORREF color;          /* outline color of the legend symbol, and
                                   the color of the text */

       LOGBRUSH brush;          // fill color for the legend symbol 

      };

//***************************************************************************

               //**  LINE/AREA CHART PARAMETER STRCTURES **
               //====================================

/*****************************************************************************
   The parameter structure definition that is used by your application
   to define x or y axes of a line chart.
*****************************************************************************/
 struct StrLineAxis {
       BOOL     draw;           // TRUE to draw the axis 

       int      orig;           /* Origin value for the axis in your
                                   application unit */
       int      max;            /* Rightmost value for the X axis or
                                   the topmost value for the Y axis.  
                                   This value is specified in your application
                                   unit.
                                   This value must be greater than 'min'.

                                   If you do not wish to draw the higher
                                   side of the axis, specify max=orig. */

       int      min;            /* Leftmost value for the X axis or
                                   the bottommost value for the Y axis.  
                                   This value is specified in your application
                                   unit.
                                   This value must be less than 'max'.
                                   
                                   If you do not wish to draw the lower 
                                   side of the axis, specify min=orig. */
                                    
       COLORREF color;          // color of the  axis line 
       int      depth;          // axis plane depth in mm, use -1 for default 
       
       int      Interval1;      /* Major axis mark interval in application 
                                   unit */
       int      Interval2;      /* Minor axis mark interval in application 
                                   unit */
       int      Len1;           // length of major axis marks in mm 
       int      Len2;           // length of minor axis marks in mm 
       COLORREF Color1,         // Color to draw the major marks 
                Color2;         // Color to draw the minor marks 
       BOOL     PrintMajorValue;// print the value for the major marks 

       char     title[50];      // title for the axis 
       int      TitleX;         /* X position (mm) of the axis title relative 
                                   to the left chart position.  Specify -1 to 
                                   use the default. The default value centers 
                                   the title of the X axis with bottom margin,
                                   or centers the title of the Y axis 
                                   within left margin */
       int      TitleY;         /* Y position (mm) of the title relative to the 
                                   top chart position.  Specify -1 to use
                                   the default.  The default value centers 
                                   the title of the X axis within bottom margin,
                                   or centers the title of the Y axis
                                   within left margin */
       COLORREF TitleColor;     // color of the axis title 

       HFONT    hFont;          /* Font used to write the axis title, specify 
                                   0 to use default font.*/

       };    

/*****************************************************************************
   This parameter structure defines the line chart title fields.  
*****************************************************************************/
 struct StrLineTitle {
       char     text[60];       /* Chart title. If the text length is
                                   zero, the corresponding title is
                                   not printed.*/
       int      x;              /* X position (mm) of the title relative to the 
                                   top left chart position.  Specify -1 to 
                                   center horizontally */
       int      y;              /* Y position (mm) of the title relative to the 
                                   top left chart position.  Specify -1 to use
                                   the default.  The default value positions
                                   the main title centered within the
                                   top margin, and positions the subtitle
                                   below the main title. */

       COLORREF TextColor;      // Foreground color for the title 
       BOOL     opaque;         /* Print title with opaque background, specify
                                   the background color using the BackColor 
                                   field */ 
       COLORREF BackColor;      /* Background color for the title, needed only
                                   when 'opaque' is TRUE */
       HFONT    hFont;          /* Font used to write the title, specify 0
                                   to use the default font. The default font
                                   for the main title is 18 point size.
                                   The default font for the
                                   subtitle is 14 point size. */
       };

/*****************************************************************************
   The parameter structure definition that is used by your application
   to specify the general line chart parameters.
   Use the 'InitLineHdr' function to initialize this structure before 
   assigning your application values.
*****************************************************************************/
 struct StrLineHdr { 
       HDC      hDC;            /* Display context of the output window or 
                                   printer */

       HANDLE   hInst;          // Handle of the current instanaces. 

       BOOL     show;           // Set to TRUE to show the picture on the device 
       BOOL     ReturnBitmap;   /* Set to TRUE to return the handle of the 
                                   chart bitmap compatible with the display
                                   device */
       HBITMAP  hBM;            /* (output) The DLL returns the chart bitmap
                                   handle when ReturnBitmap is TRUE */
       
               //** Chart Position/Size Parameters **
       int      UnitType;       /* ChartPro uses millimeters (mm) to specify 
                                   many chart dimension parameters.  You can
                                   set the millimeter unit to logical (UNIT_LOGICAL)
                                   or physical (UNIT_PHYSICAL). The default
                                   is UNIT_PHYSICAL */

       int      x;              /* Top-left X position of the line chart in 
                                   device units */
       int      y;              /* Top-left Y position of the line char in 
                                   device units */
       int      width;          // Width of the line chart in mm (millimeters) 
       int      height;         // Height of the line chart in mm (millimeters) 

       int      PosFlag;        /* LINE_HORZ=x axis specifies the positions and
                                             the y axis specifies the values.
                                   LINE_VERT=y axis specifies the positions and
                                             the x axis specifies the values.*/

       int      HorzAngle;      // Horizontal view angle (-60 to 60). 
       int      VertAngle;      // Vertical view angle (-60 to 60). 

       BOOL     area;           /* when true the area between the line and
                                   the position axis is painted */

                      //** Chart Margins **
       int      LeftMargin,     // Left Margin in mm (millimeter) 
                RightMargin,    // Right Margin in mm 
                TopMargin,      // Top Margin in mm 
                BotMargin;      // Bottom margin in mm 

               //** Chart background Parameters **
       BOOL     ClearBackground; /* TRUE=To clear background, FALSE=leave the 
                                   background as it is. The following two 
                                   values must be specified when ClearBackground
                                   is TRUE*/
       COLORREF ChartBackground, // Background color for the entire chart 
                LineBackground;  // Background colors behind the lines 

       int      LabelSize;       /* Point size of the text to label the axis
                                    and line values (1 point = 1/72 inches ) */
       char     LabelTypeface[32]; // font type face for labelling 

               //********** Chart Titles ********
       struct  StrLineTitle main,// main title information 
                            sub; // subtitle information 

               //********** Chart Axes ********
       struct  StrLineAxis ax,   // information about the x axis 
                           ay;   // information about the y axis 
       
       BOOL (CALLBACK _export *AxisValueLabel)(char,int,LPSTR); /* Process instance
                                   of the routine to provide axis value labels.
                                   You can specify NULL for this parameter
                                   if you wish the DLL to create axis value
                                   labels automatically */  
                       
               //** Line parameters **
       int      LineCount;        // number of lines to display 
       struct   StrLine far *line;/* pointer to a structure containing 
                                     parameters for each line.*/

               //** Legend parameters **
       int      LegLoc;         /* Legend box location: 
                                   LEG_HIDE, LEG_TOP, LEG_BOT, LEG_LEFT,
                                   LEG_RIGHT, LEG_TOP_LEFT, LEG_TOP_RITGHT,
                                   LEG_BOT_LEFT, LEG_BOT_RIGHT,
                                   and LEG_SPECIFY */

       int      LegX,           /* x position (mm) of the legend box relative
                                   to the chart box (when LegLoc==LEG_SPECIFY) */

                LegY;           /* y position (mm) of the legend box with
                                   respect to the chart box (when LegLoc ==
                                   LEG_SPECIFY) */

       COLORREF LegColor;       // legend box color 

       int      LegCount;       // number of legends to display. 

       struct   StrLineLegend far *legend;/* pointer to a structure containing 
                                   each legend description.*/

                // Subobject location information OUTPUT by the DLL;
       struct   StrSubObj far *subobj;  /* this pointer should be initialized
                                           to NULL before calling for the first
                                           time.  The DLL returns the x/y locations
                                           of the chart subobjects in the memory block
                                           pointed by this pointer.  On subsequent
                                           calls, this pointer should be left
                                           at the value returned by the DLL */
       int      SubobjCount;            /* number of subobjects in the location
                                           pointed by the subobj pointer variable */
       };

/*****************************************************************************
   The parameter structure definition that is used by your application
   to specify the individual line parameters.
   Use the 'InitLine' function to initialize this structure for every line before 
   assigning your application values.
*****************************************************************************/
 struct StrLine { 
       int      PointCount;     // number of points in the line 

       int      flag;           /* Position Flags.  This flag specifies how
                                   'pos', 'PosZ',z, and depth fields are 
                                   interpreted:
                                   POS_ABS   = Absolute Positioning.
                                               Positions according to the your
                                               application value in the
                                               'pos' and 'value' array fields.
                                               The 'PosZ' field specifies the
                                               Z axis placement.
                                   POS_BEHIND= Position behind the previous 
                                               line. The 'PosZ' field is ignored.
                                               The 'pos' and 'value' fields
                                               are used as in POS_ABS.             
                                   POS_STACK = Position on top of the 
                                               previous line.  When POS_STACK
                                               is selected, the 'pos','z', 
                                               and 'depth' fields are
                                               ignored, and the corresponding
                                               values are taken from the previous
                                               line. The PointCount field
                                               must be equal or smaller than
                                               the PointCount field of the
                                               previous line. */

       int      pos[MAX_LINE_POINTS]; /* specifies the position of the line
                                   points when the 'flag' field is equal to 
                                   POS_ABS or POS_BEHIND.The position is 
                                   specified using your application unit. 
                                    
                                   If PosFlag (see StrLineHdr) is equal to
                                   LINE_HORZ, the 'pos' must be within
                                   'max' and 'min' values for the X axis.
                                   When PosFlag is equal to LINE_VERT, the
                                   'pos' must be within 'max' and 'min' values
                                   for the Y axis.
                                   
                                   The PointCount field indicates the number
                                   of items used in the array*/

       int      value[MAX_LINE_POINTS];/* specifies the value of the line points
                                   in your application unit. When PosFlag 
                                   (see StrLineHdr) is equal to LINE_HORZ, 
                                   the 'value' must be within 'max' and 'min'
                                   values for the Y axis. Similarly, when 
                                   PosFlag is equal to LINE_VERT, the 'value' 
                                   must be within 'max' and 'min' values for
                                   the X axis 

                                   The PointCount field indicates the number
                                   of items used in the array*/

       int      PosZ;           /* z axis position for the line. This field
                                   is ignored when flag is equal to POS_BEHIND
                                   or POS_STACK. 
                                   Non-zero value is visible only when 
                                   non-zero view angles are used */

       int      depth;          /* depth of the line in mm (millimeters). This
                                   field is ignored when flag is equal to
                                   POS_STACK.  
                                   The view angles (see StrLineHdr) must be 
                                   non-zero to actually see the line depth.*/

       LOGBRUSH FrontBrush,     // brush to paint the front side of the area 
                SideBrush;      // brush to paint other sides.

       COLORREF LineColor;      // color of the line 
       int      LineStyle;      // PS_SOLID,PS_DASH,PS_DOT,PS_DASHDOT
       int      LineWidth;      // line width in 1/10 of mm 


       BOOL     solid;          /* TRUE to draw solid area, otherwise only
                                   the outlines are drawn */

       char     desc[30];       /* Line description.  To omit the line 
                                   description, specify a zero length 
                                   descriptoin. 
                                   Line description should be used sparingly
                                   to avoid crowded appearance */
       int      DescPoint;      /* Point where the description begins.  This
                                   value must be between 0 and PointCount */
       COLORREF DescColor;      // color used to print the description 
       BOOL     DescBox;        /* Show the description in a description box,
                                   otherwise the description appears over
                                   the line */
       COLORREF DescBoxColor;   // color of the description box                              

       int      symbol;         /* Symbol printed to indicate the points.
                                   SYM_NONE= don't print a symbol
                                   SYM_CIRCLE= circle shape
                                   SYM_RECT=rectangle shape
                                   SYM_DIAMOND=diamond shape
                                   SYM_TRI=triangle shape
                                   SYM_CHAR=a character given by SymChar field
                                   is printed.  */

        char    SymChar;         /* character to print to indicate a point when
                                   LineSymbol = SYM_CHAR.  The character is
                                   printed using the label font */

       };

/*****************************************************************************
   The parameter structure defines each legend used by the line charts.
   Use the 'InitLineLegend' function to initialize this structure for 
   every legend before assigning your application values.
*****************************************************************************/
 struct StrLineLegend { 
       char     desc[30];       // Legend description.  

       int      symbol;         /* Symbol printed for the legend
                                   SYM_CIRCLE= circle shape
                                   SYM_RECT=rectangle shape
                                   SYM_DIAMOND=diamond shape
                                   SYM_TRI=triangle shape
                                   SYM_CHAR=a character given by SymChar field
                                            is printed.
                                   SYM_LINE=a line specified by LineWidth, and
                                   LineStyle fields are printed */

       char     SymChar;        /* character to print to indicate a point when
                                   LineSymbol = SYM_CHAR.  The character is
                                   printed using the label font */

       int      LineWidth;      /* width of line when LegSymbol=SYM_LINE. The
                                   width is specified in 1/10 of mm */

       int      LineStyle;      // PS_SOLID,PS_DASH,PS_DOT,PS_DASHDOT

       
       COLORREF color;          /* outline color of the legend symbol, and
                                   the color of the text */

       LOGBRUSH brush;          // fill color for the legend symbol 

      };
//***************************************************************************

               //**  HLB (HiLo Bar) CHART PARAMETER STRCTURES **
               //====================================

/*****************************************************************************
   The parameter structure definition that is used by your application
   to define x or y axes of a hlb chart.
*****************************************************************************/
 struct StrHlbAxis {
       BOOL     draw;           // TRUE to draw the axis 

       int      orig;           /* Origin value for the axis in your
                                   application unit */
       int      max;            /* Rightmost value for the X axis or
                                   the topmost value for the Y axis.  
                                   This value is specified in your application
                                   unit.
                                   This value must be greater than 'min'.

                                   If you do not wish to draw the higher
                                   side of the axis, specify max=orig. */

       int      min;            /* Leftmost value for the X axis or
                                   the bottommost value for the Y axis.  
                                   This value is specified in your application
                                   unit.
                                   This value must be less than 'max'.
                                   
                                   If you do not wish to draw the lower 
                                   side of the axis, specify min=orig. */
                                    
       COLORREF color;          // color of the  axis line 
       int      depth;          // axis plane depth in mm, use -1 for default 
       
       int      Interval1;      /* Major axis mark interval in application 
                                   unit */
       int      Interval2;      /* Minor axis mark interval in application 
                                   unit */
       int      Len1;           // length of major axis marks in mm 
       int      Len2;           // length of minor axis marks in mm 
       COLORREF Color1,         // Color to draw the major marks 
                Color2;         // Color to draw the minor marks 
       BOOL     PrintMajorValue;// print the value for the major marks 

       char     title[50];      // title for the axis 
       int      TitleX;         /* X position (mm) of the axis title relative 
                                   to the left chart position.  Specify -1 to 
                                   use the default. The default value centers 
                                   the title of the X axis with bottom margin,
                                   or centers the title of the Y axis 
                                   within left margin */
       int      TitleY;         /* Y position (mm) of the title relative to the 
                                   top chart position.  Specify -1 to use
                                   the default.  The default value centers 
                                   the title of the X axis within bottom margin,
                                   or centers the title of the Y axis
                                   within left margin */
       COLORREF TitleColor;     // color of the axis title 

       HFONT    hFont;          /* Font used to write the axis title, specify 
                                   0 to use default font.*/

       };    

/*****************************************************************************
   This parameter structure defines the hlb (HiLo Bar) chart title fields.  
*****************************************************************************/
 struct StrHlbTitle {
       char     text[60];       /* Chart title. If the text length is
                                   zero, the corresponding title is
                                   not printed.*/
       int      x;              /* X position (mm) of the title relative to the 
                                   top left chart position.  Specify -1 to 
                                   center horizontally */
       int      y;              /* Y position (mm) of the title relative to the 
                                   top left chart position.  Specify -1 to use
                                   the default.  The default value positions
                                   the main title centered within the
                                   top margin, and positions the subtitle
                                   below the main title. */

       COLORREF TextColor;      // Foreground color for the title 
       BOOL     opaque;         /* Print title with opaque background, specify
                                   the background color using the BackColor 
                                   field */ 
       COLORREF BackColor;      /* Background color for the title, needed only
                                   when 'opaque' is TRUE */
       HFONT    hFont;          /* Font used to write the title, specify 0
                                   to use the default font. The default font
                                   for the main title is 18 point size.
                                   The default font for the
                                   subtitle is 14 point size. */
       };

/*****************************************************************************
   The parameter structure definition that is used by your application
   to specify the general hlb (HiLo Bar) chart parameters. 
   Use the 'InitHlbHdr' function to initialize this structure before 
   assigning your application values.
*****************************************************************************/

 struct StrHlbHdr { 
       HDC      hDC;            /* Display context of the output window or 
                                   printer */

       HANDLE   hInst;          // Handle of the current instanaces. 

       BOOL     show;           // Set to TRUE to show the picture on the device 
       BOOL     ReturnBitmap;   /* Set to TRUE to return the handle of the 
                                   chart bitmap compatible with the display
                                   device */
       HBITMAP  hBM;            /* (output) The DLL returns the chart bitmap
                                   handle when ReturnBitmap is TRUE */
       
               //** Chart Position/Size Parameters **
       int      UnitType;       /* ChartPro uses millimeters (mm) to specify 
                                   many chart dimension parameters.  You can
                                   set the millimeter unit to logical (UNIT_LOGICAL)
                                   or physical (UNIT_PHYSICAL). The default
                                   is UNIT_PHYSICAL */

       int      x;              /* Top-left X position of the hlb chart in 
                                   device units */
       int      y;              /* Top-left Y position of the hlb char in 
                                   device units */
       int      width;          // Width of the hlb chart in mm (millimeters) 
       int      height;         // Height of the hlb chart in mm (millimeters) 

       int      PosFlag;        /* HLB_HORZ=Position the bars on the X axis
                                   HLB_VERT=Position the bars on the Y axis */

       int      HorzAngle;      /* Horizontal view angle (-60 to 60). The zero
                                   angle hides the left and right sides of
                                   the bars */
       int      VertAngle;      /* Vertical view angle (-60 to 60). The zero
                                   angle hides the top and bottom sides of
                                   the bars */

                      //** Chart Margins **
       int      LeftMargin,     // Left Margin in mm (millimeter) 
                RightMargin,    // Right Margin in mm 
                TopMargin,      // Top Margin in mm 
                BotMargin;      // Bottom margin in mm 

               //** Chart background Parameters **
       BOOL     ClearBackground; /* TRUE=To clear background, FALSE=leave the 
                                   background as it is. The following two 
                                   values must be specified when ClearBackground
                                   is TRUE*/
       COLORREF ChartBackground,// Background color for the entire chart 
                HlbBackground;  // Background colors behind the bars 

       int      LabelSize;      /* Point size of the text to label the axis
                                   and hlb values (1 point = 1/72 inches ) */
       char     LabelTypeface[32]; // font type face for labelling 

               //********** Chart Titles ********
       struct  StrHlbTitle main,// main title information 
                           sub; // subtitle information 

               //********** Chart Axes ********
       struct  StrHlbAxis ax,   // information about the x axis 
                          ay;   // information about the y axis 
       
       BOOL (CALLBACK _export *AxisValueLabel)(char,int,LPSTR); /* Process instance
                                   of the routine to provide axis value labels.
                                   You can specify NULL for this parameter
                                   if you wish the DLL to create axis value
                                   labels automatically */  
                       
               //** Hlb parameters **
       int      HlbCount;       // number of bars to display 
       struct   StrHlb far *hlb;/* pointer to a structure containing 
                                   parameters for each bar.*/

               //** Legend parameters **
       int      LegLoc;         /* Legend box location: 
                                   LEG_HIDE, LEG_TOP, LEG_BOT, LEG_LEFT,
                                   LEG_RIGHT, LEG_TOP_LEFT, LEG_TOP_RITGHT,
                                   LEG_BOT_LEFT, LEG_BOT_RIGHT,
                                   and LEG_SPECIFY */

       int      LegX,           /* x position (mm) of the legend box relative
                                   to the chart box (when LegLoc==LEG_SPECIFY) */

                LegY;           /* y position (mm) of the legend box with
                                   respect to the chart box (when LegLoc ==
                                   LEG_SPECIFY) */

       COLORREF LegColor;       // legend box color 

       int      LegCount;       // number of legends to display. 

       struct   StrHlbLegend far *legend;/* pointer to a structure containing 
                                   each legend description.*/

                // Subobject location information OUTPUT by the DLL;
       struct   StrSubObj far *subobj;  /* this pointer should be initialized
                                           to NULL before calling for the first
                                           time.  The DLL returns the x/y locations
                                           of the chart subobjects in the memory block
                                           pointed by this pointer.  On subsequent
                                           calls, this pointer should be left
                                           at the value returned by the DLL */
       int      SubobjCount;            /* number of subobjects in the location
                                           pointed by the subobj pointer variable */
       };

/*****************************************************************************
   The parameter structure definition that is used by your application
   to specify the individual hlb (HiLo Bar) parameters.
   Use the 'InitHlb' function to initialize this structure for every bar before 
   assigning your application values.
*****************************************************************************/
 struct StrHlb { 
       int      pos;            /* specifies the position of the bar in
                                   your application units.
                                    
                                   If PosFlag (see StrHlbHdr) is equal to
                                   HLB_HORZ, the 'pos' must be within
                                   'max' and 'min' values for the X axis.
                                   When PosFlag is equal to HLB_VERT, the
                                   'pos' must be within 'max' and 'min' values
                                   for the Y axis.*/

       int      LowValue;       /* The low value of the high-low bar in your
                                   application unit. When PosFlag 
                                   (see StrHlbHdr) is equal to HLB_HORZ, 
                                   the 'LoValue' must be within 'max' and 'min'
                                   values for the Y axis. Similarly, when 
                                   PosFlag is equal to HLB_VERT, the 'LoValue' 
                                   must be within 'max' and 'min' values for
                                   the X axis */

       int      HighValue;      /* The high value of the high-low bar in your
                                   application unit. Refer to 'LoValue'
                                   for the description of value restrictions.*/

       int      OpenValue;      /* The value of the 'open' mark in your
                                   application unit. The 'OpenValue' must
                                   be between LowValue and HighValue inclusive */

       int      CloseValue;     /* The value of the 'close' mark in your
                                   application unit. The 'OpenValue' must
                                   be between LowValue and HighValue inclusive */

       int      PosZ;           // 'Z' position of the var in mm 

       int      OpenLen;        /* length of the 'open' indicator mark in mm.  
                                   Specify zero if you do not wish to display 
                                   the open mark */
       int      CloseLen;       /* length of the 'close' indicator mark in mm.
                                   Specify zero if you do not wish to display 
                                   the 'close' mark */
       int      OpenWidth;      // width of the 'open' indicator mark in mm 
       int      CloseWidth;     // width of the 'close' indicator mar in mm 


       LOGBRUSH FrontBrush,     // brush to paint the front side of the bar 
                SideBrush;      // brush to paint other sides.

       COLORREF OutlineColor;   // color of the outlines of the bar 

       BOOL     solid;          /* TRUE to draw solid bars, otherwise only
                                   the outlines are drawn */

       BOOL     shadow;         /* TRUE to draw shadow.  Shadowed bars don't
                                   have sides */

       int      width;          /* width of the bar. A positive number 
                                   specifies the width in millimeters.  
                                   A negative number specified width in the 
                                   application units.  The absolute of the 
                                   negative number is used. */

       int      depth;          /* dept of the bar in mm (millimeters)  
                                   The view angles (see StrHlbHdr) must be 
                                   non-zero to actually see the hlb depth. */
                                   
       char     desc[30];       /* Bar description.  To omit the bar 
                                   description, specify a zero length 
                                   descriptoin.

                                   Bar description should be used sparingly
                                   to avoid crowded appearance */

       int      DescLoc;        /* DESC_TOP=print description on top of the
                                   bar, DESC_BOT=print description undre the 
                                   bar */ 
       COLORREF DescColor;      // color used to print the description 
       BOOL     DescBox;        /* Show the description in a description box,
                                   otherwise the description appears under
                                   the hlb */
       COLORREF DescBoxColor;   // color of the description box                              


       int      ShowLabel;      /* Specify one or more flag to indicate which
                                   values to show:
                                   SHOW_LOW:   Print the low value
                                   SHOW_HIGH:  Print the high value
                                   SHOW_OPEN:  Print the open mark value
                                   SHOW_CLOSE: Print the close mark value

                                   Use the '|' operator use specify more than
                                   one value, eg SHOW_LOW|SHOW_OPEN. */

       char     LowLabel[10];   /* specify a value label to print for the
                                   low value.  If a zero length label is 
                                   specified, a label is constructed using 
                                   the 'LowValue' field */
       char     HighLabel[10];  /* specify a value label to print for the
                                   high value.  If a zero length label is 
                                   specified, a label is constructed using 
                                   the 'HighValue' field */
       char     OpenLabel[10];  /* specify a value label to print for the
                                   'open' mark value.  If a zero length label is 
                                   specified, a label is constructed using 
                                   the 'OpenValue' field */
       char     CloseLabel[10]; /* specify a value label to print for the
                                   'close' mark value.  If a zero length label is 
                                   specified, a label is constructed using 
                                   the 'CloseValue' field */

       COLORREF LabelColor;      // color used to print the labels 
       };

/*****************************************************************************
   The parameter structure defines each legend used by the hlb charts.
   Use the 'InitHlbLegend' function to initialize this structure for 
   every legend before assigning your application values.
*****************************************************************************/
 struct StrHlbLegend { 
       char     desc[30];       // Legend description.  
       
       COLORREF color;          /* outline color of the legend symbol, and
                                   the color of the text */

       LOGBRUSH  brush;         // brush to fill the legend symbol 

      };


//***************************************************************************

               //** 3D POINT CHART PARAMETER STRCTURES **
               //====================================

/*****************************************************************************
   The parameter structure definition that is used by your application
   to define x, y or z axes of a point chart.
*****************************************************************************/
 struct StrPointAxis {
       BOOL     draw;           // TRUE to draw the axis 

       int      orig;           /* Origin value for the axis in your
                                   application unit */
       int      max;            /* Rightmost value for the X axis or
                                   the topmost value for the Y axis or
                                   deepmost value for the Z axis.  
                                   This value is specified in your application
                                   unit.
                                   This value must be greater than 'min'.

                                   If you do not wish to draw the higher
                                   side of the axis, specify max=orig. */

       int      min;            /* Leftmost value for the X axis or
                                   the bottommost value for the Y axis or
                                   outmost value for the Z axis
                                     
                                   This value is specified in your application
                                   unit.
                                   This value must be less than 'max'.
                                   
                                   If you do not wish to draw the lower 
                                   side of the axis, specify min=orig. */
                                    
       COLORREF color;          // color of the  axis line
        
       int      Interval1;      /* Major axis mark interval in application 
                                   unit */
       int      Interval2;      /* Minor axis mark interval in application 
                                   unit */
       int      Len1;           // length of major axis marks in mm 
       int      Len2;           // length of minor axis marks in mm 
       COLORREF Color1,         // Color to draw the major marks 
                Color2;         // Color to draw the minor marks 
       BOOL     PrintMajorValue;// print the value for the major marks 

       char     title[50];      // title for the axis 
       int      TitleX;         /* X position (mm) of the axis title relative 
                                   to the left chart position.  Specify -1 to 
                                   use the default. The default value centers 
                                   the title of the X axis with bottom margin,
                                   or centers the title of the Y axis 
                                   within left margin */
       int      TitleY;         /* Y position (mm) of the title relative to the 
                                   top chart position.  Specify -1 to use
                                   the default.  The default value centers 
                                   the title of the X axis within bottom margin,
                                   or centers the title of the Y axis
                                   within left margin */
       COLORREF TitleColor;     // color of the axis title 

       HFONT    hFont;          /* Font used to write the axis title, specify 
                                   0 to use default font.*/

       };    

/*****************************************************************************
   This parameter structure defines the point chart title fields.  
*****************************************************************************/
 struct StrPointTitle {
       char     text[60];       /* Chart title. If the text length is
                                   zero, the corresponding title is
                                   not printed.*/
       int      x;              /* X position (mm) of the title relative to the 
                                   top left chart position.  Specify -1 to 
                                   center horizontally */
       int      y;              /* Y position (mm) of the title relative to the 
                                   top left chart position.  Specify -1 to use
                                   the default.  The default value positions
                                   the main title centered within the
                                   top margin, and positions the subtitle
                                   below the main title. */

       COLORREF TextColor;      // Foreground color for the title 
       BOOL     opaque;         /* Print title with opaque background, specify
                                   the background color using the BackColor 
                                   field */ 
       COLORREF BackColor;      /* Background color for the title, needed only
                                   when 'opaque' is TRUE */
       HFONT    hFont;          /* Font used to write the title, specify 0
                                   to use the default font. The default font
                                   for the main title is 18 point size.
                                   The default font for the
                                   subtitle is 14 point size. */
       };

/*****************************************************************************
   The parameter structure definition that is used by your application
   to specify the general point chart parameters.
   Use the 'InitPointHdr' function to initialize this structure before 
   assigning your application values.
*****************************************************************************/
 struct StrPointHdr { 
       HDC      hDC;            /* Display context of the output window or 
                                   printer */

       HANDLE   hInst;          // Handle of the current instanaces. 

       BOOL     show;           // Set to TRUE to show the picture on the device 
       BOOL     ReturnBitmap;   /* Set to TRUE to return the handle of the 
                                   chart bitmap compatible with the display
                                   device */
       HBITMAP  hBM;            /* (output) The DLL returns the chart bitmap
                                   handle when ReturnBitmap is TRUE */
       
               //** Chart Position/Size Parameters **
       int      UnitType;       /* ChartPro uses millimeters (mm) to specify 
                                   many chart dimension parameters.  You can
                                   set the millimeter unit to logical (UNIT_LOGICAL)
                                   or physical (UNIT_PHYSICAL). The default
                                   is UNIT_PHYSICAL */

       int      x;              /* Top-left X position of the point chart in 
                                   device units */
       int      y;              /* Top-left Y position of the point char in 
                                   device units */
       int      width;          // Width of the point chart in mm (millimeters) 
       int      height;         // Height of the point chart in mm (millimeters) 

       int      HorzAngle;      // Horizontal view angle (-60 to 60). 
       int      VertAngle;      // Vertical view angle (-60 to 60). 


                      //** Chart Margins **
       int      LeftMargin,     // Left Margin in mm (millimeter) 
                RightMargin,    // Right Margin in mm 
                TopMargin,      // Top Margin in mm 
                BotMargin;      // Bottom margin in mm 

               //** Chart background Parameters **
       BOOL     ClearBackground; /* TRUE=To clear background, FALSE=leave the 
                                   background as it is. The following two 
                                   values must be specified when ClearBackground
                                   is TRUE*/
       COLORREF ChartBackground; // Background color for the entire chart 

       int      LabelSize;       /* Point size of the text to label the axis
                                    and points (1 point = 1/72 inches ) */
       char     LabelTypeface[32]; // font type face for labelling 

               //********** Chart Titles ********
       struct  StrPointTitle main,// main title information 
                             sub; // subtitle information 

               //********** Chart Axes ********
       struct  StrPointAxis ax,   // information about the x axis 
                            ay,   // information about the y axis 
                            az;   // information about the z axis 
       
       COLORREF AxisPlaneColor;   // color of the axex planes. The axis
                                  // planes are colored solid only for 
                                  // a single-quadrangle chart.

       int      ZAxisWidth;       // width of the Z axis in mm
                                  // set to -1 to let the DLL calculate
                                  // automatically

       BOOL (CALLBACK _export *AxisValueLabel)(char,int,LPSTR); /* Process instance
                                   of the routine to provide axis value labels.
                                   You can specify NULL for this parameter
                                   if you wish the DLL to create axis value
                                   labels automatically */  
                       
               //** Point parameters **
       int      PointCount;        // number of points to display 
       struct   StrPoint far *point;/* pointer to a structure containing 
                                     parameters for each point.*/

               //** Label Parameters **
       int      LabelCount;        // number of miscellaneous labels
       struct   StrPointLabel far *label; // pointer to a structure containing
                                          // parameters for each label

               //** Legend parameters **
       int      LegLoc;         /* Legend box location: 
                                   LEG_HIDE, LEG_TOP, LEG_BOT, LEG_LEFT,
                                   LEG_RIGHT, LEG_TOP_LEFT, LEG_TOP_RITGHT,
                                   LEG_BOT_LEFT, LEG_BOT_RIGHT,
                                   and LEG_SPECIFY */

       int      LegX,           /* x position (mm) of the legend box relative
                                   to the chart box (when LegLoc==LEG_SPECIFY) */

                LegY;           /* y position (mm) of the legend box with
                                   respect to the chart box (when LegLoc ==
                                   LEG_SPECIFY) */

       COLORREF LegColor;       // legend box color 

       int      LegCount;       // number of legends to display. 

       struct   StrPointLegend far *legend;/* pointer to a structure containing 
                                   each legend description.*/

       };

/*****************************************************************************
   The parameter structure definition that is used by your application
   to specify the individual point parameters.
   Use the 'InitPoint' function to initialize this structure for every point before 
   assigning your application values.
*****************************************************************************/
 struct StrPoint { 
       int      x,              // specifies the x,y,z positions of the point.
                y,              // These values must be within the minmum
                z;              // and maximum values for the corresponding
                                // axis.
       
       BOOL     LineType;       // Control connection of current point to 
                                // the previous point
                                // LINE_NONE     : suppress line
                                // LINE_STRAIGHT : connect using a straight line
                                // LINE_SMOOTH   : connect using a smooth circular
                                //                 approximation

       COLORREF LineColor;      // color of the connecting line 
       int      LineStyle;      // line style of the connecting line
                                //   PS_SOLID,PS_DASH,PS_DOT,PS_DASHDOT
       int      LineWidth;      // line width in 1/10 of mm of the connecting line

       int      symbol;         /* Symbol printed to indicate the points.
                                   SYM_NONE= don't print a symbol
                                   SYM_CIRCLE= circle shape
                                   SYM_RECT=rectangle shape
                                   SYM_DIAMOND=diamond shape
                                   SYM_TRI=triangle shape
                                   SYM_CHAR=a character given by SymChar field
                                   is printed.  */

        char    SymChar;         /* character to print to indicate a point when
                                   PointSymbol = SYM_CHAR.  The character is
                                   printed using the label font */
       
        LOGBRUSH SymBrush;       // fill color for the symbol 
                         
             /* (OUTPUT ONLY) Physical Point location returned by ChartPro */
        int     OutX;            /* X location of the point relative to 
                                    the chart top/left in pixel units */
        int     OutY;            /* Y location of the point relative to 
                                    the chart top/left in pixel units */
       };

/*****************************************************************************
   The parameter structure defines each label used by the point charts.
   Use the 'InitPointLabel' function to initialize this structure for 
   every legend before assigning your application values.
*****************************************************************************/
 struct StrPointLabel { 
       char     desc[30];       // label text  
       int      point;          // The 'point' where this label is printed
                                // The 'point' must be between 0 and PointCount-1
                                // The x,y,z location of the specified point
                                // provides the text location.

       int      distance;       // distance in mm from the given point
       int      angle;          // angle at which the text is drawn

       COLORREF DescColor;      // color used to print the text 
       BOOL     DescBox;        // Show the description in a description box.
                                // The 'distance' and 'angle' fields are ignored
                                // when 'DescBox' is TRUE. 
       COLORREF DescBoxColor;   // color of the description box                              
      
      };

/*****************************************************************************
   The parameter structure defines each legend used by the point charts.
   Use the 'InitPointLegend' function to initialize this structure for 
   every legend before assigning your application values.
*****************************************************************************/
 struct StrPointLegend { 
       char     desc[30];       // Legend description.  

       int      symbol;         /* Symbol printed for the legend
                                   SYM_CIRCLE= circle shape
                                   SYM_RECT=rectangle shape
                                   SYM_DIAMOND=diamond shape
                                   SYM_TRI=triangle shape
                                   SYM_CHAR=a character given by SymChar field
                                            is printed.
                                   SYM_LINE=a line specified by LineWidth, and
                                   LineStyle fields are printed */

       char     SymChar;        /* character to print to indicate a point when
                                   PointSymbol = SYM_CHAR.  The character is
                                   printed using the label font */

       int      LineWidth;      /* width of line when LegSymbol=SYM_LINE. The
                                   width is specified in 1/10 of mm */

       int      LineStyle;      // PS_SOLID,PS_DASH,PS_DOT,PS_DASHDOT

       
       COLORREF color;          /* outline color of the legend symbol, and
                                   the color of the text */

       LOGBRUSH brush;          // fill color for the legend symbol 

      };

/*****************************************************************************
    Object position structure array.  This array is used by the DLLs to return
    the position of the chart subobjects.  A chart object such as a bar consists
    of many subobjects.  Ex. a bar consists of 6 subobjects which are the six
    planes that make a 3d bar.
*****************************************************************************/
 struct StrSubObj { 
       int   object;            // object number that this subobject belongs to
       char  type;              // subobject type, SOBJ_POLYGON,SOBJ_ELLIPSE,SOBJ_ARC
       int   PointCount;        // number of points used in the point array below
       POINT pt[4];             // up to 4 points to define the subobject
      };

/****************************************************************************
   Instructions to resume the default structure packing.
*****************************************************************************/
#ifndef RC_INVOKED
   #pragma pack()
#endif

/*****************************************************************************
   Calleable Function Prototypes
   When calling the APIs from a C program, leave the CHW_CPLUSPLUS symbol 
   undefined. When calling APIs from a C++ program, define the 
   CHW_CPLUSPLUS as following:

   #define CHW_CPLUSPLUS extern "C"
   #include "chart.h"
*****************************************************************************/
#ifndef CHW_CPLUSPLUS
   #if defined(__cplusplus)
      #define CHW_CPLUSPLUS extern "C"
   #else
      #define CHW_CPLUSPLUS
   #endif
#endif

CHW_CPLUSPLUS  int  WINAPI _export ShowBar(struct StrBarHdr far *);
CHW_CPLUSPLUS  BOOL WINAPI _export EditBar(struct StrBarHdr far *);
CHW_CPLUSPLUS  int  WINAPI _export LocateBar(struct StrBarHdr far *,int,int);
CHW_CPLUSPLUS  BOOL WINAPI _export InitBarHdr(struct StrBarHdr far *);
CHW_CPLUSPLUS  BOOL WINAPI _export InitBarLegend(struct StrBarLegend far *);
CHW_CPLUSPLUS  BOOL WINAPI _export InitBar(struct StrBar far *);
CHW_CPLUSPLUS  int FAR PASCAL _export SaveBarToFile(struct StrBarHdr far *,LPBYTE);
CHW_CPLUSPLUS  BOOL WINAPI _export BarSetLicenseKey(LPBYTE);


CHW_CPLUSPLUS  int  WINAPI _export ShowPie(struct StrPieHdr far *);
CHW_CPLUSPLUS  BOOL WINAPI _export EditPie(struct StrPieHdr far *);
CHW_CPLUSPLUS  int  WINAPI _export LocatePie(struct StrPieHdr far *,int,int);
CHW_CPLUSPLUS  BOOL WINAPI _export InitPieHdr(struct StrPieHdr far *);
CHW_CPLUSPLUS  BOOL WINAPI _export InitPieLegend(struct StrPieLegend far *);
CHW_CPLUSPLUS  BOOL WINAPI _export InitPie(struct StrPie far *);
CHW_CPLUSPLUS  int FAR PASCAL _export SavePieToFile(struct StrPieHdr far *,LPBYTE);
CHW_CPLUSPLUS  BOOL WINAPI _export PieSetLicenseKey(LPBYTE);

CHW_CPLUSPLUS  int  WINAPI _export ShowLine(struct StrLineHdr far *);
CHW_CPLUSPLUS  BOOL WINAPI _export EditLine(struct StrLineHdr far *);
CHW_CPLUSPLUS  int  WINAPI _export LocateLine(struct StrLineHdr far *,int,int);
CHW_CPLUSPLUS  BOOL WINAPI _export InitLineHdr(struct StrLineHdr far *);
CHW_CPLUSPLUS  BOOL WINAPI _export InitLineLegend(struct StrLineLegend far *);
CHW_CPLUSPLUS  BOOL WINAPI _export InitLine(struct StrLine far *);
CHW_CPLUSPLUS  int FAR PASCAL _export SaveLineToFile(struct StrLineHdr far *,LPBYTE);
CHW_CPLUSPLUS  BOOL WINAPI _export LineSetLicenseKey(LPBYTE);

CHW_CPLUSPLUS  int  WINAPI _export ShowHlb(struct StrHlbHdr far *);
CHW_CPLUSPLUS  BOOL WINAPI _export EditHlb(struct StrHlbHdr far *);
CHW_CPLUSPLUS  int  WINAPI _export LocateHlb(struct StrHlbHdr far *,int,int);
CHW_CPLUSPLUS  BOOL WINAPI _export InitHlbHdr(struct StrHlbHdr far *);
CHW_CPLUSPLUS  BOOL WINAPI _export InitHlbLegend(struct StrHlbLegend far *);
CHW_CPLUSPLUS  BOOL WINAPI _export InitHlb(struct StrHlb far *);
CHW_CPLUSPLUS  int FAR PASCAL _export SaveHlbToFile(struct StrHlbHdr far *,LPBYTE);
CHW_CPLUSPLUS  BOOL WINAPI _export HlbSetLicenseKey(LPBYTE);

CHW_CPLUSPLUS  int  WINAPI _export ShowPoint(struct StrPointHdr far *);
CHW_CPLUSPLUS  BOOL WINAPI _export EditPoint(struct StrPointHdr far *);
CHW_CPLUSPLUS  BOOL WINAPI _export LocatePoint(struct StrPointHdr far *,int,int,int);
CHW_CPLUSPLUS  BOOL WINAPI _export InitPointHdr(struct StrPointHdr far *);
CHW_CPLUSPLUS  BOOL WINAPI _export InitPointLegend(struct StrPointLegend far *);
CHW_CPLUSPLUS  BOOL WINAPI _export InitPointLabel(struct StrPointLabel far *);
CHW_CPLUSPLUS  BOOL WINAPI _export InitPoint(struct StrPoint far *);
CHW_CPLUSPLUS  int FAR PASCAL _export SavePointToFile(struct StrPointHdr far *,LPBYTE);
CHW_CPLUSPLUS  BOOL WINAPI _export PntSetLicenseKey(LPBYTE);


/****************************************************************************
               Alternate Library Function Prototypes
     These function provide handle based interface to ChartPro DLLs.
*****************************************************************************/
CHW_CPLUSPLUS BOOL  WINAPI _export cDeleteChart(int);
CHW_CPLUSPLUS BOOL  WINAPI _export cEditChart(int);
CHW_CPLUSPLUS HANDLE WINAPI _export cGetBitmap(int hChart);
CHW_CPLUSPLUS BOOL  WINAPI _export cGetChartData(int,int DataType,int,void far *);
CHW_CPLUSPLUS int   WINAPI _export cGetChartDataLen(int,int);
CHW_CPLUSPLUS int   WINAPI _export cObjectFromPoint(int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetDevice(int,HDC);
CHW_CPLUSPLUS int   WINAPI _export cCreateChart(int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cReqBitmap(int,BOOL,BOOL);
CHW_CPLUSPLUS BOOL  WINAPI _export cResetAxisLabel(int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetAngle(int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetAreaShadow(int,BOOL);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetAxisBknd(int,COLORREF);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetAxisLabel(int,int,int,LPSTR);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetAxisLabelProc(int,FARPROC);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetAxisLimits(int,int,int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetAxisMark(int,int,int,int,int,COLORREF);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetAxisMisc(int,int,BOOL,COLORREF,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetAxisTitle(int,int,LPSTR,int,int,COLORREF,HFONT);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetBarValue(int,int,int,int,int,BOOL);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetBarValueLabel(int,int,int,int,LPSTR);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetChartBknd(int,BOOL,COLORREF);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetChartData(int,int,int,void far *);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetHlbLabel(int,int,int,COLORREF);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetHlbLabelText(int,int,LPSTR,LPSTR,LPSTR,LPSTR);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetHlbTab(int,int,int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetHlbValue(int,int,int,int,int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetItemDesc(int,int,LPSTR,COLORREF,BOOL,COLORREF);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetItemDepth(int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetItemOutlineColor(int,int,COLORREF);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetItemPosZ(int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetItemShadow(int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetItemSolid(int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetItemSurfaceBrush(int,int,int,LOGBRUSH far *);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetItemSurfaceColor(int,int,int,COLORREF);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetItemWidth(int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetLabelFont(int,LPSTR,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetLegBox(int,int,int,int,COLORREF);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetLegBrush(int,int,LOGBRUSH far *);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetLegColor(int,int,COLORREF);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetLegSymbol(int,int,int,char,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetLegText(int,int,LPSTR,COLORREF);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetLineDescPoint(int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetLinePoint(int,int,int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetLinePointCount(int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetLinePosFlag(int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetLineSymbol(int,int,int,char);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetLineType(int,int,COLORREF,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetMargin(int,int,int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetPos(int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cPrintAxisValueLabels(int,int,BOOL);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetSize(int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetObjectPlacement(int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetTitleText(int,int,LPSTR,int,int,HFONT);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetTitleColor(int,int,COLORREF,BOOL,COLORREF);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetUnit(int,int);
CHW_CPLUSPLUS int   WINAPI _export cShowChart(int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetPieShow(int,int,BOOL);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetPieValue(int,int,int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetPieValueLabel(int,int,BOOL,int,LPSTR);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetPointLabelCount(int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetPointZAxisWidth(int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetPoint(int,int,int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetPointLine(int,int,int,COLORREF,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetPointSymbol(int,int,int,char);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetPointLabel(int,int,LPSTR,int,int,int);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetPointLabelColor(int,int,COLORREF,BOOL,COLORREF);
CHW_CPLUSPLUS int   WINAPI _export cGetAxisValue(int,int,int);
CHW_CPLUSPLUS int   WINAPI _export cSaveChartToFile(int,LPBYTE);
CHW_CPLUSPLUS BOOL  WINAPI _export cSetLicenseKey(LPBYTE);

