/*==============================================================================
   DEMO.C :- BAR Char Demo Program (MS Windows Version)
   Sub Systems, Inc.
   Software License Agreement  (1993)              
   ----------------------------------
   This is a demo program.  The program shows how you can utilize the Bar
   Chart routines in your application.
===============================================================================*/

#include "windows.h"

#if defined (_WIN32)
   #if !defined(WIN32)
     #define WIN32
   #endif
#endif
#if !defined(WIN32) 
  #include "print.h"
#endif

#include "stdio.h"
#include "stdlib.h"
#include "fcntl.h"
#include "sys\types.h"
#include "sys\stat.h"
#include "io.h"
#include "dos.h"
#include "string.h"
#include "math.h"

#if defined(__TURBOC__)
   #include "dir.h"
   #include "mem.h"
#else
   #include "memory.h"
#endif

/******************************************************************************
    WIN32 specific defines
*******************************************************************************/
#if defined (WIN32)
   #undef  huge
   #define huge
   #undef  FreeProcInstance 
   #define FreeProcInstance(x) 
   #define COMMAND_ID(wp,lp) LOWORD(wp)
   #define THUMB_POS(wp,lp)  HIWORD(wp)
   #define farmalloc  malloc
   #define farrealloc realloc
   #define farfree    free
#else
   #define COMMAND_ID(wp,lp) (wp)
   #define THUMB_POS(wp,lp)  LOWORD(lp)
#endif

#include "chart.h"
#include "demo.h"

/****************************************************************************
        Main Fuction
******************************************************************************/
int PASCAL WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
    MSG msg;
    int i;

    hPrevInst=hPrevInstance;
    if (!hPrevInstance) if (!InitApplication(hInstance))return (FALSE);
    if (!InitInstance(hInstance, nCmdShow))return (FALSE);
    
    InitBarData();           // initialize the bar graph structure 
    InitPieData();           // initialize the pie graph structure 
    InitLineData();          // initialize the line graph structure 
    InitHlbData();           // initialize the High-low bar graph structure 
    InitPointData();         // initialize the 3D Point graph structure 

    ShowChart(TYPE_BAR);     // show the bar chart 

    while (GetMessage(&msg, NULL, 0, 0)) {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
    
    return (msg.wParam);
}

/****************************************************************************
   Initialize window data and register window class
****************************************************************************/

BOOL InitApplication(HANDLE hInstance)
{
    WNDCLASS  wc;

    wc.style = CS_DBLCLKS|CS_OWNDC|CS_HREDRAW|CS_VREDRAW;    // allocate a DC for this class 
    wc.lpfnWndProc = (void far *)DemoWndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH); 
    wc.lpszMenuName =  "DemoMenu";
    wc.lpszClassName = "ChartProClass";

    return (RegisterClass(&wc));
}


/****************************************************************************
    Save instance handle and create main window
****************************************************************************/
BOOL InitInstance(HANDLE hInstance,int nCmdShow)
{
    hInst    = hInstance;
    ChartType=TYPE_NONE;        // hold painting until parameters are set 

    hDemoWin = CreateWindow(    // create the demo window 
        "ChartProClass",
        "ChartPro Demo",
        WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        NULL,
        NULL,
        hInstance,
        NULL
    );

    if (!hDemoWin)
        return (FALSE);

    ShowWindow(hDemoWin, nCmdShow);
    
    if (NULL==(hDC=GetDC(hDemoWin))) {     // allocate a class DC 
         MessageBox(NULL,"Unable to allocate a Display Context.",NULL,MB_OK);
         return FALSE;
    }

    //********** calculate screen resolution ************
    //PixPerInchX=(GetDeviceCaps(hDC,HORZRES)*25)/GetDeviceCaps(hDC,HORZSIZE);
    //PixPerInchY=(GetDeviceCaps(hDC,VERTRES)*25)/GetDeviceCaps(hDC,VERTSIZE);
    
    //Uncomment these two lines to select logical units (preferable when writing to a metafile) 
    //also set UnitType=UNIT_LOGICAL in chart header structure
    PixPerInchX=GetDeviceCaps(hDC,LOGPIXELSX);
    PixPerInchY=GetDeviceCaps(hDC,LOGPIXELSY);
    
    hWaitCursor=LoadCursor(NULL,IDC_WAIT); // load hour glass cursor 

    StockHlbCount=20;
    pHlb=malloc(sizeof(struct StrHlb)*StockHlbCount);

    SetChartTimer();        // display different charts on a timer

    return TRUE;
}

/****************************************************************************
    InitBarData:
    Initialize the variables in the bar parameter structures.
    The demo program shows profit vs. revenue bar chart for
    1990 through 1994.  The revenue is split into sales and service
    components.  The 1994 bars are drawn hollow to indicate estimated
    data.
****************************************************************************/
BOOL InitBarData()
{
    int  i;

    //****** Initialize the StrBarHdr structure variables *********
    InitBarHdr(&BarHdr);       /* assign the default values to the structure
                                  variable. 
                                  
                                  This step allows you to change only the
                                  variables that you would like to have
                                  non-default values.*/ 
    
                                  
    BarHdr.hDC     =hDC;       /* Display context of the output window or 
                                  printer */
    BarHdr.hInst   =hInst;     // Handle of the current instanaces. 


    BarHdr.UnitType=UNIT_LOGICAL;  // use logical units to convert pixels to mm

    //** Chart Position/Size Parameters **
    BarHdr.x=10;               /* Top-left X position of the bar chart in 
                                  device units */
    BarHdr.y=10;               /* Top-left Y position of the bar char in 
                                  device units */
    BarHdr.width=130;          // Width of the bar chart in mm (millimeters) 
    BarHdr.height=100;         // Height of the bar chart in mm (millimeters) 

    BarHdr.HorzAngle=20;       /* Horizontal view angle (-60 to +60). The zero
                                  angle hides the left and right sides of
                                  the bars */
    BarHdr.VertAngle=25;       /* Vertical view angle (-60 to +60). The zero
                                  angle hides the top and bottom sides of
                                  the bars */

    //** Title Parameters **
    strcpy(BarHdr.main.text,"Demo Bar Graph");     // Main title 
    strcpy(BarHdr.sub.text,"5 Year Growth Trend"); // Subtitle 

    //** Set X Axis parameters **
    BarHdr.ax.orig=1990;       /* beginning value for the X axis in your
                                  application unit */
    BarHdr.ax.max=1995;        /* Right (typically positive) value for the X 
                                  axis in your application unit */
    BarHdr.ax.min=1990;        /* Left (typically negative) value for the X 
                                  axis in your application unit. If you
                                  do not need the left side of the graph,
                                  specify xAxisLeft=xAxisOrig */

    BarHdr.ax.Interval1=1;     /* Major X axis mark interval in application 
                                  unit */
    BarHdr.ax.Interval2=0;     /* Minor X axis mark interval in application 
                                  unit */
    BarHdr.ax.Len1=4;          // length of major X axis marks in mm 
    BarHdr.ax.Len2=0;          // length of minor X axis marks in mm 
    
    strcpy(BarHdr.ax.title,"Years"); // title for the x axis 

    //** Set Y Axis parameters **
    BarHdr.ay.orig=10;         /* beginning value for the Y axis in your
                                  application unit */
    BarHdr.ay.max=80;          /* Top (typically positive) value for the Y 
                                  axis in your application unit */
    BarHdr.ay.min=10;          /* Bottom (typically negative) value for the Y 
                                  axis in your application unit. If you 
                                  do not need the bottom side of the graph,
                                  specify yAxisBot=yAxisOrig */
    BarHdr.ay.Interval1=20;    /* Major Y axis mark interval in application 
                                  unit */
    BarHdr.ay.Interval2=10;    /* Minor Y axis mark interval in application 
                                  unit */
    BarHdr.ay.Len1=4;          // length of major Y axis marks in mm 
    BarHdr.ay.Len2=2;          // length of minor Y axis marks in mm 

    strcpy(BarHdr.ay.title,"Dollars"); // title for the Y axis 

    /** Specify a routine that will provide that value of the axis labels.  You
        can specify NULL for AxisValueLabel parameter if you wish the DLL to
        generate labels automatically */
    BarHdr.AxisValueLabel=(void far *)MakeProcInstance(BarAxisValueLabel,hInst); 

    //** Bar parameters **
    BarHdr.BarCount=15;        // number of bars to display 
    BarHdr.bar=bar;            /* pointer to a structure containing 
                                  parameters for each bar. see demo.h */

    //*** legend box ***
    BarHdr.LegCount=3;          // number of legend lines in the legend box 
    BarHdr.legend=BarLeg;       /* pointer to the legend line structure.
                                   see demo.h. The StrLegend structure is
                                   filled later in this function */

              /********************************************* 
                            Initialize each bar 
              **********************************************/

    for (i=0;i<BarHdr.BarCount;i++) {
       InitBar(&(bar[i]));    /* Initialize the bar strcuture for 
                                 the current bar.
                                 Now assign the customize variables.*/
       
       if (i>=12)bar[i].solid=FALSE;// hollow bars for 1994 figures 

       if (i==3) strcpy(bar[i].desc,"Up 50%");      // bar description 

       if (i==12) strcpy(bar[i].desc,"Projected");  // bar description 
       if (i==12) bar[i].DescBox=FALSE;             // don't show bar description in a box 

       if (i>=12)bar[i].PrintValue=FALSE;// don't print bar values for 1994

    }

              /************************************************* 
                       Initialize each bar specificaton 
              This example attepts to assign varying specifications
              to each bar to show different aspects of bar drawing.
              **************************************************/

    //********* specify the bar colors *********
    for (i=0;i<BarHdr.BarCount;i=i+3) {  // specify the PROFIT bars 
       bar[i].FrontBrush.lbColor=Yellow; // brush to paint the front side of the bar 
       bar[i].SideBrush.lbColor=YellowLt;// brush to paint other sides 
    }
    for (i=1;i<BarHdr.BarCount;i=i+3) {  // specify the SALES revenue bars 
       bar[i].FrontBrush.lbColor=Green;  // brush to paint the front side of the bar 
       bar[i].SideBrush.lbColor=GreenLt; // brush to paint other sides 
    }
    for (i=2;i<BarHdr.BarCount;i=i+3) {  // specify the SERVICE revenue bars 
       bar[i].FrontBrush.lbColor=Magenta;// brush to paint the front side of the bar 
       bar[i].SideBrush.lbColor=MagentaLt;// brush to paint other sides 
    }
    
    // specify the bar position and value 
    i=0;bar[i].flag=POS_ABS;   bar[i].pos=1990;bar[i].value=20; // profit 1990 
    i++;bar[i].flag=POS_BEHIND;bar[i].pos=0;   bar[i].value=30; // sales 1990 
    i++;bar[i].flag=POS_STACK; bar[i].pos=0;   bar[i].value=10; // service 1990 

    i++;bar[i].flag=POS_ABS;   bar[i].pos=1991;bar[i].value=30; // profit 1991 
    i++;bar[i].flag=POS_BEHIND;bar[i].pos=0;   bar[i].value=40; // sales 1991 
    i++;bar[i].flag=POS_STACK; bar[i].pos=0;   bar[i].value=25; // service 1991 

    i++;bar[i].flag=POS_ABS;   bar[i].pos=1992;bar[i].value=35; // profit 1992 
    i++;bar[i].flag=POS_BEHIND;bar[i].pos=0;   bar[i].value=45; // sales 1992 
    i++;bar[i].flag=POS_STACK; bar[i].pos=0;   bar[i].value=25; // service 1992 

    i++;bar[i].flag=POS_ABS;   bar[i].pos=1993;bar[i].value=38; // profit 1993 
    i++;bar[i].flag=POS_BEHIND;bar[i].pos=0;   bar[i].value=45; // sales 1993 
    i++;bar[i].flag=POS_STACK; bar[i].pos=0;   bar[i].value=31; // service 1993 

    //*** show last three bars hollow with different outline colors ***
    i++;bar[i].flag=POS_ABS;   bar[i].pos=1994;bar[i].value=45; bar[i].OutlineColor=Yellow;// profit 1994 
    i++;bar[i].flag=POS_BEHIND;bar[i].pos=0;   bar[i].value=40; bar[i].OutlineColor=Green;// sales 1994 
    i++;bar[i].flag=POS_STACK; bar[i].pos=0;   bar[i].value=31; bar[i].OutlineColor=Magenta;// service 1994 


              /********************************************* 
                        Initialize each legend line 
              **********************************************/
    for (i=0;i<3;i++) InitBarLegend(&(BarLeg[i])); // initialize the legend structure 

    i=0;BarLeg[i].brush.lbColor=Yellow; strcpy(BarLeg[i].desc,"Profit");
    i++;BarLeg[i].brush.lbColor=Green;  strcpy(BarLeg[i].desc,"Sales Revenue");
    i++;BarLeg[i].brush.lbColor=Magenta;strcpy(BarLeg[i].desc,"Service Revenue");

    SelectedBar=-1;            // hilighted bar using the mouse

    return TRUE;
}

/****************************************************************************
    InitPieData:
    Initialize the variables in the pie parameter structures.
    The demo program shows the gross allocation of the federal budget using
    a pie chart.
****************************************************************************/
BOOL InitPieData()
{
    int  i;

    //****** Initialize the StrPieHdr structure variables *********
    InitPieHdr(&PieHdr);       /* assign the default values to the structure
                                  variable. 
                                  
                                  This step allows you to change only the
                                  variables that you would like to have
                                  non-default values.*/ 
                                  
    PieHdr.hDC     =hDC;       /* Display context of the output window or 
                                  printer */
    PieHdr.hInst   =hInst;     // Handle of the current instanaces. 


    //** Chart Position/Size Parameters **
    PieHdr.x=10;               /* Top-left X position of the pie chart in 
                                  device units */
    PieHdr.y=10;               /* Top-left Y position of the pie char in 
                                  device units */
    PieHdr.width=130;          // Width of the pie chart in mm (millimeters) 
    PieHdr.height=100;         // Height of the pie chart in mm (millimeters) 

    PieHdr.InitAngle=10;       /* Angle placement of the first pie.  The 0
                                  value begins the first pie at the positive
                                  x axis.  The successive pies will be
                                  places counter clockwise */
    PieHdr.VertAngle=50;       /* Vertical view angle (0 to 60). The zero
                                  angle hides the depth of the pies */

    //** Title Parameters **
    strcpy(PieHdr.main.text,"Demo Pie Graph");     // Main title 
    strcpy(PieHdr.sub.text,"Federal Budget Allocation");   // Subtitle 

    //** Pie parameters **
    PieHdr.PieCount=5;         // number of pies to display 
    PieHdr.pie=pie;            /* pointer to a structure containing 
                                  parameters for each pie. see demo.h */

    //*** legend box ***
    PieHdr.LegCount=4;          // number of legend lines in the legend box 
    PieHdr.legend=PieLeg;       /* pointer to the legend line structure.
                                   see demo.h. The StrLegend structure is
                                   filled later in this function */

              /********************************************* 
                            Initialize each pie 
              **********************************************/

    i=0;
    InitPie(&(pie[i]));              // Initialize the pie strcuture 
    strcpy(pie[i].desc,"Defence");   // pie description 
    pie[i].flag=POS_CENTER;          // position the pie at the origin 
    pie[i].value=280;                // value that this pie represents 
    pie[i].FrontBrush.lbColor=Green; // brush to paint the front side of the pie 
    pie[i].SideBrush.lbColor=GreenLt;// brush to paint the height of the pie 
    pie[i].DescColor=Green;          // color of the description label 

    i=1;
    InitPie(&(pie[i]));              // Initialize the pie strcuture 
    strcpy(pie[i].desc,"Education"); // pie description 
    pie[i].flag=POS_CENTER;          // position the pie at the origin 
    pie[i].value=80;                 // value that this pie represents 
    pie[i].FrontBrush.lbColor=Yellow;// brush to paint the front side of the pie 
    pie[i].SideBrush.lbColor=YellowLt;// brush to paint the height of the pie 
    pie[i].DescColor=Yellow;         // color of the description label 

    i=2;
    InitPie(&(pie[i]));              // Initialize the pie strcuture 
    strcpy(pie[i].desc,"Medicare");  // pie description 
    pie[i].flag=POS_CENTER;          // position the pie at the origin 
    pie[i].value=480;                // value that this pie represents 
    pie[i].FrontBrush.lbColor=Cyan;  // brush to paint the front side of the pie 
    pie[i].SideBrush.lbColor=CyanLt; // brush to paint the height of the pie 
    pie[i].DescColor=Cyan;           // color of the description label 

    i=3;
    InitPie(&(pie[i]));              // Initialize the pie strcuture 
    strcpy(pie[i].desc,"Deficit");   // pie description 
    pie[i].DescBox=TRUE;             // show description in a box 
    pie[i].flag=POS_OUT;             // position the pie at the origin 
    pie[i].pos=5;                    // 5 mm pulled out 
    pie[i].value=310;                // value that this pie represents 
    pie[i].solid=FALSE;              // show outline only 
    pie[i].FrontBrush.lbColor=Blue;  // brush to paint the front side of the pie 
    pie[i].SideBrush.lbColor=BlueLt; // brush to paint the height of the pie 
    pie[i].DescColor=Blue;           // color of the description label 
    pie[i].OutlineColor=Red;         // outline color 

    i=4;
    InitPie(&(pie[i]));              // Initialize the pie strcuture 
    strcpy(pie[i].desc,"Other");     // pie description 
    pie[i].flag=POS_CENTER;          // position the pie at the origin 
    pie[i].value=220;                // value that this pie represents 
    pie[i].FrontBrush.lbColor=Magenta;// brush to paint the front side of the pie 
    pie[i].SideBrush.lbColor=MagentaLt;// brush to paint the height of the pie 
    pie[i].DescColor=Magenta;        // color of the description label 

              /********************************************* 
                        Initialize each legend line 
              **********************************************/
    for (i=0;i<4;i++) InitPieLegend(&(PieLeg[i])); // initialize the legend structure 

    i=0;PieLeg[i].brush.lbColor=Green;  strcpy(PieLeg[i].desc,"Defence");
    i++;PieLeg[i].brush.lbColor=Yellow; strcpy(PieLeg[i].desc,"Education");
    i++;PieLeg[i].brush.lbColor=Cyan;   strcpy(PieLeg[i].desc,"Medicare");
    i++;PieLeg[i].brush.lbColor=Magenta;strcpy(PieLeg[i].desc,"Other");

    SelectedPie=-1;            // currently hilighted pie wedge using the mouse

    //pie[0].value=45;           // test
    //pie[1].value=45;           // test
    //pie[2].value=45;           // test
    //pie[3].value=45;           // test
    //pie[4].value=180;           // test
    //pie[3].solid=TRUE;         // test
    //PieHdr.VertAngle=60;

    return TRUE;
}

/****************************************************************************
    InitLineData:
    Initialize the variables in the line parameter structures.
    The demo program shows profit vs. revenue line chart for
    1990 through 1995.  The revenue is split into sales and service
    components.
****************************************************************************/
BOOL InitLineData()
{
    int  i;

    //****** Initialize the StrLineHdr structure variables *********
    InitLineHdr(&LineHdr);       /* assign the default values to the structure
                                    variable. 
                                  
                                    This step allows you to change only the
                                    variables that you would like to have
                                    non-default values.*/ 
                                  
    LineHdr.hDC     =hDC;       /* Display context of the output window or 
                                   printer */
    LineHdr.hInst   =hInst;     // Handle of the current instanaces. 


    LineHdr.UnitType=UNIT_LOGICAL;  // use logical units to convert pixels to mm

    //** Chart Position/Size Parameters **
    LineHdr.x=10;               /* Top-left X position of the line chart in 
                                   device units */
    LineHdr.y=10;               /* Top-left Y position of the line char in 
                                   device units */
    LineHdr.width=130;          // Width of the line chart in mm (millimeters) 
    LineHdr.height=100;         // Height of the line chart in mm (millimeters) 

    LineHdr.HorzAngle=20;       /* Horizontal view angle (-60 to +60). The zero
                                   angle hides the left and right sides of
                                   the lines */
    LineHdr.VertAngle=25;       /* Vertical view angle (-60 to +60). The zero
                                   angle hides the top and bottom sides of
                                   the lines */

    //** Title Parameters **
    strcpy(LineHdr.main.text,"Demo Line Graph");     // Main title 
    strcpy(LineHdr.sub.text,"6 Year Growth Trend"); // Subtitle 

    //** Set X Axis parameters **
    LineHdr.ax.orig=1990;       /* beginning value for the X axis in your
                                   application unit */
    LineHdr.ax.max=1995;        /* Right (typically positive) value for the X 
                                   axis in your application unit */
    LineHdr.ax.min=1990;        /* Left (typically negative) value for the X 
                                   axis in your application unit. If you
                                   do not need the left side of the graph,
                                   specify xAxisLeft=xAxisOrig */

    LineHdr.ax.Interval1=1;     /* Major X axis mark interval in application 
                                   unit */
    LineHdr.ax.Interval2=0;     /* Minor X axis mark interval in application 
                                   unit */
    LineHdr.ax.Len1=4;          // length of major X axis marks in mm 
    LineHdr.ax.Len2=0;          // length of minor X axis marks in mm 
    
    strcpy(LineHdr.ax.title,"Years"); // title for the x axis 

    //** Set Y Axis parameters **
    LineHdr.ay.orig=10;         /* beginning value for the Y axis in your
                                   application unit */
    LineHdr.ay.max=80;          /* Top (typically positive) value for the Y 
                                   axis in your application unit */
    LineHdr.ay.min=10;          /* Bottom (typically negative) value for the Y 
                                   axis in your application unit. If you 
                                   do not need the bottom side of the graph,
                                   specify yAxisBot=yAxisOrig */
    LineHdr.ay.Interval1=20;    /* Major Y axis mark interval in application 
                                   unit */
    LineHdr.ay.Interval2=10;    /* Minor Y axis mark interval in application 
                                   unit */
    LineHdr.ay.Len1=4;          // length of major Y axis marks in mm 
    LineHdr.ay.Len2=2;          // length of minor Y axis marks in mm 

    strcpy(LineHdr.ay.title,"Dollars"); // title for the Y axis 

    /** Specify a routine that will provide that value of the axis labels.  You
        can specify NULL for AxisValueLabel parameter if you wish the DLL to
        generate labels automatically */
    LineHdr.AxisValueLabel=(void far *)MakeProcInstance(LineAxisValueLabel,hInst); 

    //** Line parameters **
    LineHdr.LineCount=3;          // number of lines to display 
    LineHdr.line=line;            /* pointer to a structure containing 
                                     parameters for each line. see demo.h */

    //*** legend box ***
    LineHdr.LegCount=3;          // number of legend lines in the legend box 
    LineHdr.legend=LineLeg;      /* pointer to the legend line structure.
                                    see demo.h. The StrLineLegend structure is
                                    filled later in this function */

              /********************************************* 
                            Initialize each line 
              **********************************************/

    for (i=0;i<LineHdr.LineCount;i++) {
       InitLine(&(line[i]));    /* Initialize the line strcuture for 
                                   the current line.
                                   Now assign the customize variables.*/
       
    }

              /************************************************* 
                       Initialize each line specificaton 
              This example attepts to assign varying specifications
              to each line to show different aspects of line drawing.
              **************************************************/
    i=0;                                   // define first line 
    InitLine(&(line[i]));                  // Initialize the line strcuture
    line[i].flag=POS_ABS;                  // absolute position given for each point 
    line[i].FrontBrush.lbColor=Yellow;     // brush to paint the front side of the line 
    line[i].SideBrush.lbColor=YellowLt;    // brush to paint other sides 
    line[i].symbol=SYM_CIRCLE;
    lstrcpy(line[i].desc,"Up 50%");        // show a description label 
    line[i].DescPoint=1;                   // show description at the second point 

    line[i].PosZ=5;                        // z position of the line (mm) 
    line[i].PointCount=6;                  // define 6 points
    line[i].pos[0]=1990; line[i].value[0]=20; // line position and values 
    line[i].pos[1]=1991; line[i].value[1]=30;
    line[i].pos[2]=1992; line[i].value[2]=35;
    line[i].pos[3]=1993; line[i].value[3]=38;
    line[i].pos[4]=1994; line[i].value[4]=38;
    line[i].pos[5]=1995; line[i].value[5]=45;

    i++;                                   // define next line 
    InitLine(&(line[i]));                  // Initialize the line strcuture
    line[i].flag=POS_BEHIND;               // position 2nd line behind the first one 
    line[i].FrontBrush.lbColor=Green;      // brush to paint the front side of the line 
    line[i].SideBrush.lbColor=GreenLt;     // brush to paint other sides 
    line[i].symbol=SYM_RECT;

    line[i].PointCount=6;                  // define 6 points
    line[i].pos[0]=1990; line[i].value[0]=30; // line position and values 
    line[i].pos[1]=1991; line[i].value[1]=40;
    line[i].pos[2]=1992; line[i].value[2]=45;
    line[i].pos[3]=1993; line[i].value[3]=45;
    line[i].pos[4]=1994; line[i].value[4]=40;
    line[i].pos[5]=1995; line[i].value[5]=48;


    i++;                                   // define next line 
    InitLine(&(line[i]));                  // Initialize the line strcuture
    line[i].flag=POS_STACK;                // position 3rd line stacked on the 2nd line 
    line[i].FrontBrush.lbColor=Magenta;    // brush to paint the front side of the line 
    line[i].SideBrush.lbColor=MagentaLt;   // brush to paint other sides 
    line[i].symbol=SYM_DIAMOND;
    
    line[i].PointCount=6;                  // define 6 points
    line[i].value[0]=10;                   // do not specify pos for stacked line 
    line[i].value[1]=25;
    line[i].value[2]=25;
    line[i].value[3]=31;
    line[i].value[4]=31;
    line[i].value[5]=25;

              /********************************************* 
                        Initialize each legend line 
              **********************************************/
    for (i=0;i<3;i++) InitLineLegend(&(LineLeg[i])); // initialize the legend structure 

    i=0;LineLeg[i].symbol=SYM_CIRCLE; LineLeg[i].brush.lbColor=Yellow; strcpy(LineLeg[i].desc,"Profit");
    i++;LineLeg[i].symbol=SYM_RECT;   LineLeg[i].brush.lbColor=Green;  strcpy(LineLeg[i].desc,"Sales Revenue");
    i++;LineLeg[i].symbol=SYM_DIAMOND;LineLeg[i].brush.lbColor=Magenta;strcpy(LineLeg[i].desc,"Service Revenue");


    SelectedLine=-1;            // hilighted line using the mouse

    return TRUE;
}

/****************************************************************************
    InitHlbData:
    Initialize the variables in the high-low bar parameter structures.
    The demo program shows the stock prices for a company.  The stock
    prices are given for January to June months.
****************************************************************************/
BOOL InitHlbData()
{
    int  i;

    //****** Initialize the StrHlbHdr structure variables *********
    InitHlbHdr(&HlbHdr);       /* assign the default values to the structure
                                  variable. 
                                  
                                  This step allows you to change only the
                                  variables that you would like to have
                                  non-default values.*/ 
                                  
    HlbHdr.hDC     =hDC;       /* Display context of the output window or 
                                  printer */
    HlbHdr.hInst   =hInst;     // Handle of the current instanaces. 


    HlbHdr.UnitType=UNIT_LOGICAL;  // use logical units to convert pixels to mm

    //** Chart Position/Size Parameters **
    HlbHdr.x=10;               /* Top-left X position of the hlb chart in 
                                  device units */
    HlbHdr.y=10;               /* Top-left Y position of the hlb char in 
                                  device units */
    HlbHdr.width=130;          // Width of the hlb chart in mm (millimeters) 
    HlbHdr.height=100;         // Height of the hlb chart in mm (millimeters) 

    HlbHdr.HorzAngle=20;       /* Horizontal view angle (-60 to +60). The zero
                                  angle hides the left and right sides of
                                  the bars */
    HlbHdr.VertAngle=25;       /* Vertical view angle (-60 to +60). The zero
                                  angle hides the top and bottom sides of
                                  the bars */

    //** Title Parameters **
    strcpy(HlbHdr.main.text,"Demo High/Low Graph");     // Main title 
    strcpy(HlbHdr.sub.text,"Stock Price Analysis"); // Subtitle 

    //** Set X Axis parameters **
    HlbHdr.ax.orig=0;          /* beginning value for the X axis in your
                                  application unit */
    HlbHdr.ax.max=7;           /* Right (typically positive) value for the X 
                                  axis in your application unit */
    HlbHdr.ax.min=0;           /* Left (typically negative) value for the X 
                                  axis in your application unit. If you
                                  do not need the left side of the graph,
                                  specify xAxisLeft=xAxisOrig */

    HlbHdr.ax.Interval1=1;     /* Major X axis mark interval in application 
                                  unit */
    HlbHdr.ax.Interval2=0;     /* Minor X axis mark interval in application 
                                  unit */
    HlbHdr.ax.Len1=4;          // length of major X axis marks in mm 
    HlbHdr.ax.Len2=0;          // length of minor X axis marks in mm 
    
    strcpy(HlbHdr.ax.title,"1993 ->"); // title for the x axis 

    //** Set Y Axis parameters **
    HlbHdr.ay.orig=10;         /* beginning value for the Y axis in your
                                  application unit */
    HlbHdr.ay.max=70;          /* Top (typically positive) value for the Y 
                                  axis in your application unit */
    HlbHdr.ay.min=10;          /* Bottom (typically negative) value for the Y 
                                  axis in your application unit. If you 
                                  do not need the bottom side of the graph,
                                  specify yAxisBot=yAxisOrig */
    HlbHdr.ay.Interval1=20;    /* Major Y axis mark interval in application 
                                  unit */
    HlbHdr.ay.Interval2=10;    /* Minor Y axis mark interval in application 
                                  unit */
    HlbHdr.ay.Len1=4;          // length of major Y axis marks in mm 
    HlbHdr.ay.Len2=2;          // length of minor Y axis marks in mm 

    strcpy(HlbHdr.ay.title,"Price"); // title for the Y axis 

    /** Specify a routine that will provide that value of the axis labels.  You
        can specify NULL for AxisValueLabel parameter if you wish the DLL to
        generate labels automatically */
    HlbHdr.AxisValueLabel=(void far *)MakeProcInstance(HlbAxisValueLabel,hInst); 

    //** Hlb parameters **
    HlbHdr.HlbCount=6;         // number of bars to display 
    HlbHdr.hlb=hlb;            /* pointer to a structure containing 
                                  parameters for each hlb. see demo.h */

    //*** legend box ***
    HlbHdr.LegCount=1;         // number of legend lines in the legend box 
    HlbHdr.legend=HlbLeg;      /* pointer to the legend line structure.
                                  see demo.h. The StrLegend structure is
                                  filled later in this function */

              /********************************************* 
                       Initialize each high-low bar
              The bars from 0 to 11 will be used for the first company and
              bars 12 to 23 will be used for the second company.          
              **********************************************/

    for (i=0;i<HlbHdr.HlbCount;i++) {
       InitHlb(&(hlb[i]));               /* Initialize the hlb strcuture for 
                                            the current hlb.
                                            Now assign the customize variables.*/
       
       hlb[i].width=6;                           // width of the bars 
       hlb[i].depth=6;                           // depth of the bars 
       hlb[i].PosZ=3;                            // Z position of the bars 

       hlb[i].pos=i+1;                           // specify month numbers between 1 and 12 

       if (i==5)  strcpy(hlb[i].desc,"Merger");  // bar description 
       
       hlb[i].ShowLabel=SHOW_LOW|SHOW_HIGH;      // value labels to show 
       if (i==0) hlb[i].ShowLabel=hlb[i].ShowLabel|SHOW_OPEN;  // show the open value for the first bar 
       if (i==5) hlb[i].ShowLabel=hlb[i].ShowLabel|SHOW_CLOSE; // show the close value for the first bar 

       if (i>0)  hlb[i].OpenLen=0;               // show the open marker only for the first bar 

       hlb[i].FrontBrush.lbColor=Magenta;        // brush to paint the front side of the hlb 
       hlb[i].SideBrush.lbColor=MagentaLt;       // brush to paint other sides 
    }

              /************************************************* 
                       Initialize each bar specificaton 
              This example attepts to assign varying specifications
              to each bar to show different aspects of hlb drawing.
              **************************************************/

    // specify the bar values for the first company 
    i=0;hlb[i].OpenValue=21;hlb[i].LowValue=20;hlb[i].HighValue=35;hlb[i].CloseValue=30;
    i++;hlb[i].OpenValue=30;hlb[i].LowValue=25;hlb[i].HighValue=55;hlb[i].CloseValue=40;
    i++;hlb[i].OpenValue=40;hlb[i].LowValue=30;hlb[i].HighValue=60;hlb[i].CloseValue=50;
    i++;hlb[i].OpenValue=50;hlb[i].LowValue=20;hlb[i].HighValue=50;hlb[i].CloseValue=25;
    i++;hlb[i].OpenValue=25;hlb[i].LowValue=15;hlb[i].HighValue=40;hlb[i].CloseValue=30;
    i++;hlb[i].OpenValue=30;hlb[i].LowValue=20;hlb[i].HighValue=62;hlb[i].CloseValue=50;
                                    

              /********************************************* 
                        Initialize each legend line 
              **********************************************/
    for (i=0;i<HlbHdr.LegCount;i++) InitHlbLegend(&(HlbLeg[i])); // initialize the legend structure 

    i=0;HlbLeg[i].brush.lbColor=Magenta;strcpy(HlbLeg[i].desc,"Corporate Paper Inc.");

    SelectedHlb=-1;            // hilighted bar using the mouse


    // fillup another set of hilo headers to show stock market charts
    for (i=0;i<StockHlbCount;i++) {
       InitHlb(&(pHlb[i]));               /* Initialize the hlb strcuture */
    
       pHlb[i].width=-1;                 // width in 'x' axis unit 
       pHlb[i].depth=0;                  // no depth
       pHlb[i].PosZ=0;                   // Z position of the bars 

       pHlb[i].pos=6*i+1;                // specify month numbers between 1 and 12 

       pHlb[i].OpenLen=pHlb[i].CloseLen=1;
       pHlb[i].OpenWidth=pHlb[i].CloseWidth=-1;
       pHlb[i].ShowLabel=0;
    } 
    i=-1;
    i++;SetHlbBarValue(pHlb,i,40,66,66,63);   // low,high,open,close
    i++;SetHlbBarValue(pHlb,i,57,64,64,62);
    i++;SetHlbBarValue(pHlb,i,44,60,57,44);
    i++;SetHlbBarValue(pHlb,i,40,57,44,44);
    i++;SetHlbBarValue(pHlb,i,35,43,43,35);
    i++;SetHlbBarValue(pHlb,i,37,55,50,39);
    i++;SetHlbBarValue(pHlb,i,37,52,40,38);
    i++;SetHlbBarValue(pHlb,i,35,47,38,38);
    i++;SetHlbBarValue(pHlb,i,25,42,37,37);
    i++;SetHlbBarValue(pHlb,i,21,49,25,23);
    i++;SetHlbBarValue(pHlb,i,21,36,25,36);
    i++;SetHlbBarValue(pHlb,i,17,35,25,25);
    i++;SetHlbBarValue(pHlb,i,10,27,10,27);
    i++;SetHlbBarValue(pHlb,i,16,38,21,38);
    i++;SetHlbBarValue(pHlb,i,27,37,37,30);
    i++;SetHlbBarValue(pHlb,i,21,30,30,25);
    i++;SetHlbBarValue(pHlb,i,10,27,27,14);
    i++;SetHlbBarValue(pHlb,i,14,25,14,14);
    i++;SetHlbBarValue(pHlb,i,16,25,19,19);
    i++;SetHlbBarValue(pHlb,i,18,55,25,50);


    return TRUE;
}

/****************************************************************************
    SetHlbBarValue:
    Set the values for a hilo bar.
****************************************************************************/
BOOL SetHlbBarValue(struct StrHlb far *phlb,int i, int low, int high, int open, int close)
{
    phlb[i].LowValue=low;
    phlb[i].HighValue=high;
    phlb[i].OpenValue=open;
    phlb[i].CloseValue=close;
  
    return TRUE;
}

/****************************************************************************
    InitPointData:
    Initialize the variables in the 3d Point parameter structures.
    The demo program constructs a sphere by connecting points on the 
    surface.
****************************************************************************/
BOOL InitPointData()
{
    int  i;

    //****** Initialize the StrPointHdr structure variables *********
    InitPointHdr(&PointHdr);       /* assign the default values to the structure
                                    variable. 
                                  
                                    This step allows you to change only the
                                    variables that you would like to have
                                    non-default values.*/ 
                                  
    PointHdr.hDC     =hDC;       /* Display context of the output window or 
                                   printer */
    PointHdr.hInst   =hInst;     // Handle of the current instanaces. 

    PointHdr.UnitType=UNIT_LOGICAL;  // use logical units to convert pixels to mm

    //** Chart Position/Size Parameters **
    PointHdr.x=10;               /* Top-left X position of the line chart in 
                                   device units */
    PointHdr.y=10;               /* Top-left Y position of the line char in 
                                   device units */
    PointHdr.width=130;          // Width of the line chart in mm (millimeters) 
    PointHdr.height=100;         // Height of the line chart in mm (millimeters) 

    PointHdr.HorzAngle=10;       /* Horizontal view angle (-60 to +60). The zero
                                   angle hides the left and right sides of
                                   the lines */
    PointHdr.VertAngle=15;       /* Vertical view angle (-60 to +60). The zero
                                   angle hides the top and bottom sides of
                                   the lines */

    //** Title Parameters **
    strcpy(PointHdr.main.text,"Demo Point Graph");   // Main title 
    strcpy(PointHdr.sub.text,"6 Year Growth Trend"); // Subtitle 

    //** Set X Axis parameters **
    PointHdr.ax.orig=1990;       /* beginning value for the X axis in your
                                   application unit */
    PointHdr.ax.max=1995;        /* Right (typically positive) value for the X 
                                   axis in your application unit */
    PointHdr.ax.min=1990;        /* Left (typically negative) value for the X 
                                   axis in your application unit. If you
                                   do not need the left side of the graph,
                                   specify xAxisLeft=xAxisOrig */

    PointHdr.ax.Interval1=1;     /* Major X axis mark interval in application 
                                   unit */
    PointHdr.ax.Interval2=0;     /* Minor X axis mark interval in application 
                                   unit */
    PointHdr.ax.Len1=4;          // length of major X axis marks in mm 
    PointHdr.ax.Len2=0;          // length of minor X axis marks in mm 
    
    strcpy(PointHdr.ax.title,"Years"); // title for the x axis 

    //** Set Y Axis parameters **
    PointHdr.ay.orig=10;         /* beginning value for the Y axis in your
                                   application unit */
    PointHdr.ay.max=50;          /* Top (typically positive) value for the Y 
                                   axis in your application unit */
    PointHdr.ay.min=10;          /* Bottom (typically negative) value for the Y 
                                   axis in your application unit. If you 
                                   do not need the bottom side of the graph,
                                   specify yAxisBot=yAxisOrig */
    PointHdr.ay.Interval1=20;    /* Major Y axis mark interval in application 
                                   unit */
    PointHdr.ay.Interval2=10;    /* Minor Y axis mark interval in application 
                                   unit */
    PointHdr.ay.Len1=4;          // length of major Y axis marks in mm 
    PointHdr.ay.Len2=2;          // length of minor Y axis marks in mm 

    strcpy(PointHdr.ay.title,"Dollars"); // title for the Y axis 

    //** Set X Axis parameters **
    PointHdr.az.orig=0;          /* beginning value for the Z axis in your
                                    application unit */
    PointHdr.az.max=2;           /* Max (typically positive) value for the Z 
                                    axis in your application unit */
    PointHdr.az.min=0;           /* Min (typically negative) value for the Z 
                                    axis in your application unit. If you 
                                    do not need the outer side of the graph,
                                    specify min=orig */
    PointHdr.az.Interval1=1;     /* Major Z axis mark interval in application 
                                    unit */
    PointHdr.az.Interval2=0;     /* Minor Z axis mark interval in application 
                                   unit */
    PointHdr.az.Len1=4;          // length of major Z axis marks in mm 
    PointHdr.az.Len2=2;          // length of minor Z axis marks in mm 

    strcpy(PointHdr.az.title,"Z Axis"); // title for the Z axis 

    /** Specify a routine that will provide that value of the axis labels.  You
        can specify NULL for AxisValueLabel parameter if you wish the DLL to
        generate labels automatically */
    PointHdr.AxisValueLabel=(void far *)MakeProcInstance(PointAxisValueLabel,hInst); 

    //** Point parameters **
    PointHdr.PointCount=MAX_PTS;   // number of points to display 
    PointHdr.point=point;           /* pointer to a structure containing 
                                       parameters for each point. see demo.h */

    //*** legend box ***
    PointHdr.LegCount=3;          // number of legend lines in the legend box 
    PointHdr.legend=PointLeg;     /* pointer to the legend line structure.
                                    see demo.h. The StrPointLegend structure is
                                    filled later in this function */

    //*** Point Labels ***
    PointHdr.LabelCount=2;        // number of label to write
    PointHdr.label=PointLabel;    /* pointer to the label structure.
                                    see demo.h. The StrPointLabel structure is
                                    filled later in this function */

              /********************************************* 
                            Initialize each point
               Three lines will be drawn using 6 points each.              
              **********************************************/

    for (i=0;i<PointHdr.PointCount;i++) {
       InitPoint(&(point[i]));    // Initialize the point strcuture  
    }

    // First line data - Draw farthest line first
    for (i=0;i<POINTS_PER_LINE;i++) {  // initialize common data
       point[i].z=2;                   // Use a constant Z value for the line
       point[i].LineColor=Magenta;     // color of the connecting line
       point[i].symbol=SYM_DIAMOND;    // symbols to be printed at the nodes
    }
    i=0;                               // first point of the line

    point[i+0].x=1990; point[i+0].y=30;// point position and values 
    point[i+1].x=1991; point[i+1].y=40;
    point[i+2].x=1992; point[i+2].y=45;
    point[i+3].x=1993; point[i+3].y=45;
    point[i+4].x=1994; point[i+4].y=40;
    point[i+5].x=1995; point[i+5].y=48;


    // Second line data
    for (i=POINTS_PER_LINE;i<2*POINTS_PER_LINE;i++) {  // initialize common data
       point[i].z=1;                   // Use a constant Z value for the line
       point[i].LineColor=Green;       // color of the connecting line
       point[i].symbol=SYM_RECT;       // symbols to be printed at the nodes
    }
    i=POINTS_PER_LINE;                 // first point of the line
    point[i].LineType=LINE_NONE;       // disconnect from the previous line
    
    point[i+0].x=1990; point[i+0].y=20;// point position and values 
    point[i+1].x=1991; point[i+1].y=35;
    point[i+2].x=1992; point[i+2].y=35;
    point[i+3].x=1993; point[i+3].y=41;
    point[i+4].x=1994; point[i+4].y=41;
    point[i+5].x=1995; point[i+5].y=35;

    // Third line data - draw nearest line last
    for (i=2*POINTS_PER_LINE;i<3*POINTS_PER_LINE;i++) {  // initialize common data
       point[i].z=0;                   // Use a constant Z value for the line
       point[i].LineColor=Yellow;      // color of the connecting line
       point[i].LineType=LINE_SMOOTH;  // draw this line with smooth connections
       point[i].symbol=SYM_CIRCLE;     // symbols to be printed at the nodes
    }
    i=2*POINTS_PER_LINE;               // first point of the line
    point[i].LineType=LINE_NONE;       // disconnect from the previous line

    point[i+0].x=1990; point[i+0].y=20;      // point position and values 
    point[i+1].x=1991; point[i+1].y=30;
    point[i+2].x=1992; point[i+2].y=35;
    point[i+3].x=1993; point[i+3].y=38;
    point[i+4].x=1994; point[i+4].y=38;
    point[i+5].x=1995; point[i+5].y=45;

              /************************************************* 
                       Initialize each label specificaton 
              **************************************************/
    i=0;
    InitPointLabel(&(PointLabel[i]));           // initialize the first label
    lstrcpy(PointLabel[i].desc,"Up 50%");       // show a description label 
    PointLabel[i].point=13;                     // at the second point of the third line

    i++;
    InitPointLabel(&(PointLabel[i]));           // initialize the second label
    lstrcpy(PointLabel[i].desc,"Forecast");     // show a description label 
    PointLabel[i].point=17;                     // at the last point
    PointLabel[i].DescBox=FALSE;                // don't use a tag box
    PointLabel[i].DescColor=0x0000FF;           // red color label
    PointLabel[i].angle=45;                     // text angle


              /********************************************* 
                        Initialize each legend line 
              **********************************************/
    for (i=0;i<3;i++) InitPointLegend(&(PointLeg[i])); // initialize the legend structure 

    i=0;PointLeg[i].symbol=SYM_CIRCLE; PointLeg[i].color=Yellow; strcpy(PointLeg[i].desc,"Profit");
    i++;PointLeg[i].symbol=SYM_RECT;   PointLeg[i].color=Green;  strcpy(PointLeg[i].desc,"Sales Revenue");
    i++;PointLeg[i].symbol=SYM_DIAMOND;PointLeg[i].color=Magenta;strcpy(PointLeg[i].desc,"Service Revenue");

    SelectedPoint=-1;                          // hilighted point using the mouse
    
    return TRUE;
}

/****************************************************************************
    BarAxisValueLabel:
    This routine is called by the BAR DLL before printing a label for a
    axis value.  The first argument is AXIS_X, AXIS_Y, or AXIS_Y1 to indicate 
    the axis values being labelled.  The second argument is the value of the
    marker being labelled.  This routine returns a TRUE value if it supplies 
    the value label, otherwise it returns a FALSE value.  When a TRUE value
    is returned, the label string must be copies to the pointer specified
    by the third argument. The label string including the terminating
    NULL character must not exceed 20 characters.
    On a FALSE return, the DLL automatically generates a value label 
    corresponding to the marker value.
****************************************************************************/
BOOL CALLBACK _export BarAxisValueLabel(char axis,int value,LPSTR ValueLabel)
{
    struct StrBarAxis *pAxis;

    if (axis==AXIS_X) pAxis=&(BarHdr.ax);        // pointer to the x axis parameters 
    else if (axis==AXIS_Y1) pAxis=&(BarHdr.ay1); // pointer to the second y axis parameters 
    else                    pAxis=&(BarHdr.ay);  // pointer to the primary y axis parameters 

    if ( (axis==AXIS_X && BarHdr.PosFlag==BAR_HORZ)
      || ((axis==AXIS_Y || axis==AXIS_Y1) && BarHdr.PosFlag==BAR_VERT) ){  // label years 
      if (value>=1994) {                // label the last marker 
         if (value==1994) lstrcpy(ValueLabel,"1994-95");
         else             lstrcpy(ValueLabel,"");
         return TRUE;
      }
      else return FALSE;                // let DLL label otherwise 
    }
    else {                              // label dollar axis 
      if (value+pAxis->Interval1>pAxis->max) { // label the last value 
         wsprintf(ValueLabel,"$%d",value);
         return TRUE;
      }
      else return FALSE;                // let DLL label otherwise 
    }
}

/****************************************************************************
    LineAxisValueLabel:
    This routine is called by the 'LIN' DLL before printing a label for a
    axis value.  The first argument is AXIS_X, AXIS_Y or AXIS_Y1 to indicate 
    the axis values being labelled.  The second argument is the value of the
    marker being labelled.  This routine returns a TRUE value if it supplies 
    the value label, otherwise it returns a FALSE value.  When a TRUE value
    is returned, the label string must be copies to the pointer specified
    by the third argument. The label string including the terminating
    NULL character must not exceed 20 characters.
    On a FALSE return, the DLL automatically generates a value label 
    corresponding to the marker value.
****************************************************************************/
BOOL CALLBACK _export LineAxisValueLabel(char axis,int value,LPSTR ValueLabel)
{
    struct StrLineAxis *pAxis;


    if (axis==AXIS_X) pAxis=&(LineHdr.ax);  // pointer to the x axis parameters 
    else              pAxis=&(LineHdr.ay);  // pointer to the y axis parameters 

    if ( (axis==AXIS_X && LineHdr.PosFlag==LINE_HORZ)
      || (axis==AXIS_Y && LineHdr.PosFlag==LINE_VERT) ){  // label years 
      if (value>=1995) {                // label the last marker 
         if (value==1995) lstrcpy(ValueLabel,"1995-97");
         else             lstrcpy(ValueLabel,"");
         return TRUE;
      }
      else return FALSE;                // let DLL label otherwise 
    }
    else {                              // label dollar axis 
      if (value+pAxis->Interval1>pAxis->max) { // label the last value 
         wsprintf(ValueLabel,"$%d",value);
         return TRUE;
      }
      else return FALSE;                // let DLL label otherwise 
    }
}

/****************************************************************************
    HlbAxisValueLabel:
    This routine is called by the HLB DLL before printing a label for a
    axis value.  The first argument is AXIS_X, AXIS_Y to indicate the
    axis values being labelled.  The second argument is the value of the
    marker being labelled.  This routine returns a TRUE value if it supplies 
    the value label, otherwise it returns a FALSE value.  When a TRUE value
    is returned, the label string must be copies to the pointer specified
    by the third argument. The label string including the terminating
    NULL character must not exceed 20 characters.
    On a FALSE return, the DLL automatically generates a value label 
    corresponding to the marker value.
****************************************************************************/
BOOL CALLBACK _export HlbAxisValueLabel(char axis,int value,LPSTR ValueLabel)
{
    struct StrHlbAxis *pAxis;
    char month[12][4]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep",
                       "Oct","Nov","Dec"};

    if (axis==AXIS_X) pAxis=&(HlbHdr.ax);  // pointer to the x axis parameters 
    else           pAxis=&(HlbHdr.ay);  // pointer to the y axis parameters 

    if ( (axis==AXIS_X && HlbHdr.PosFlag==HLB_HORZ)
      || (axis==AXIS_Y && HlbHdr.PosFlag==HLB_VERT) ){  // label years 
      if (value>=1 && value<=12) lstrcpy(ValueLabel,month[value-1]);   // assign month descriptor 
      else                       lstrcpy(ValueLabel,"");
      return TRUE;
    }
    else {                              // label dollar axis 
      if (value+pAxis->Interval1>pAxis->max) { // label the last value 
         wsprintf(ValueLabel,"$%d",value);
         return TRUE;
      }
      else return FALSE;                // let DLL label otherwise 
    }
}

/****************************************************************************
    PointAxisValueLabel:
    This routine is called by the 'PNT' DLL before printing a label for a
    axis value.  The first argument is AXIS_X, AXIS_Y, or AXIS_Z to indicate 
    the axis values being labelled.  The second argument is the value of the
    marker being labelled.  This routine returns a TRUE value if it supplies 
    the value label, otherwise it returns a FALSE value.  When a TRUE value
    is returned, the label string must be copies to the pointer specified
    by the third argument. The label string including the terminating
    NULL character must not exceed 20 characters.
    On a FALSE return, the DLL automatically generates a value label 
    corresponding to the marker value.
****************************************************************************/
BOOL CALLBACK _export PointAxisValueLabel(char axis,int value,LPSTR ValueLabel)
{
    struct StrPointAxis *pAxis;


    if      (axis==AXIS_X) pAxis=&(PointHdr.ax);  // pointer to the x axis parameters 
    else if (axis==AXIS_Y) pAxis=&(PointHdr.ay);  // pointer to the y axis parameters 
    else                   pAxis=&(PointHdr.az);  // pointer to the z axis parameters 

    if (axis==AXIS_X){                     // label years 
      if (value>=1995) {                // label the last marker 
         if (value==1995) lstrcpy(ValueLabel,"1995-97");
         else             lstrcpy(ValueLabel,"");
         return TRUE;
      }
      else return FALSE;                // let DLL label otherwise 
    }
    else if (axis==AXIS_Y){                // label dollar axis 
      if (value+pAxis->Interval1>pAxis->max) { // label the last value 
         wsprintf(ValueLabel,"$%d",value);
         return TRUE;
      }
      else return FALSE;                // let DLL label otherwise 
    }
    else {                              // label Z axis
      if      (value==0) lstrcpy(ValueLabel,"Profit");
      else if (value==1) lstrcpy(ValueLabel,"Sales");
      else if (value==2) lstrcpy(ValueLabel,"Services");
      else return FALSE;
    }
    return TRUE;
}


/****************************************************************************
    ShowChart:
    Display the specified chart.  This routine changes the window size
    to enclose the specified chart.
****************************************************************************/
ShowChart(int type)
{
    int WinWidth,WinHeight,ChartWidth,ChartHeight,ClientWidth,ClientHeight,result;
    RECT WinRect,ClientRect,ScrRect;
    HCURSOR hSaveCursor;

    hSaveCursor=SetCursor(hWaitCursor);         // show hour glass 

    ChartType=TYPE_NONE;                        // disable painting temporarily 

    //********** Get Current Window Dimensions *******
    GetWindowRect(GetDesktopWindow(),&ScrRect); // get the screen dimensions 
    GetWindowRect(hDemoWin,&WinRect);           // get the current window dimensions 
    WinWidth=WinRect.right-WinRect.left;
    WinHeight=WinRect.bottom-WinRect.top;

    switch (type) {
       case TYPE_BAR:     // show the bar type chart 
          
          //************* set scroll bar *********
          HorzMin=VertMin=-60;                 // allows -60 to +60 view angles 
          HorzMax=VertMax=60;
          ShowScrollBar(hDemoWin,SB_HORZ,TRUE);
          SetScrollRange(hDemoWin,SB_HORZ,HorzMin,HorzMax,FALSE);
          SetScrollPos(hDemoWin,SB_HORZ,BarHdr.HorzAngle,TRUE);
          ShowScrollBar(hDemoWin,SB_VERT,TRUE);
          SetScrollRange(hDemoWin,SB_VERT,VertMin,VertMax,FALSE);
          SetScrollPos(hDemoWin,SB_VERT,-BarHdr.VertAngle,TRUE);

          //*********** get client window size ************
          GetClientRect(hDemoWin,&ClientRect);        // get the window client area 
          ClientWidth=ClientRect.right-ClientRect.left;
          ClientHeight=ClientRect.bottom-ClientRect.top;

          // set the window size, center the chart
           
          ChartWidth=BarHdr.x+(BarHdr.width*PixPerInchX*10)/254+BarHdr.x+WinWidth-ClientWidth;
          ChartHeight=BarHdr.y+(BarHdr.height*PixPerInchY*10)/254+BarHdr.y+WinHeight-ClientHeight;
          //** center the window *
          WinRect.left=(ScrRect.right-ChartWidth)/2;
          WinRect.top=(ScrRect.bottom-ChartHeight)/2;
          MoveWindow(hDemoWin,WinRect.left,WinRect.top,ChartWidth,ChartHeight,TRUE);
          ValidateRect(hDemoWin,NULL);  // No need to repaint 

          ChartType=type;
          result=ShowBar(&BarHdr);
          break;

       case TYPE_PIE:     // show the pie type chart 
          //************** Show Scroll Pies to change rotation *****************
          HorzMin=0;                   // Minmum angle placment of the first pie 
          HorzMax=360;                 // Maximum angle placment of the first pie 
          VertMin=0;                   // allows 0 to +60 view angles 
          VertMax=60;
          ShowScrollBar(hDemoWin,SB_HORZ,TRUE);
          SetScrollRange(hDemoWin,SB_HORZ,HorzMin,HorzMax,FALSE);
          SetScrollPos(hDemoWin,SB_HORZ,PieHdr.InitAngle,TRUE);
          ShowScrollBar(hDemoWin,SB_VERT,TRUE);
          SetScrollRange(hDemoWin,SB_VERT,VertMin,VertMax,FALSE);
          SetScrollPos(hDemoWin,SB_VERT,PieHdr.VertAngle,TRUE);

          //*********** get client window size ************
          GetClientRect(hDemoWin,&ClientRect);        // get the window client area 
          ClientWidth=ClientRect.right-ClientRect.left;
          ClientHeight=ClientRect.bottom-ClientRect.top;

          // set the window size, center the chart
          ChartWidth=PieHdr.x+(PieHdr.width*PixPerInchX*10)/254+PieHdr.x+WinWidth-ClientWidth;
          ChartHeight=PieHdr.y+(PieHdr.height*PixPerInchY*10)/254+PieHdr.y+WinHeight-ClientHeight;
          //** center the window *
          WinRect.left=(ScrRect.right-ChartWidth)/2;
          WinRect.top=(ScrRect.bottom-ChartHeight)/2;
          MoveWindow(hDemoWin,WinRect.left,WinRect.top,ChartWidth,ChartHeight,TRUE);
          ValidateRect(hDemoWin,NULL);  // No need to repaint 

          ChartType=type;
          result=ShowPie(&PieHdr);
          break;

       case TYPE_LINE:     // show the line type chart 
          
          //************* set scroll bar *********
          HorzMin=VertMin=-60;                 // allows -60 to +60 view angles 
          HorzMax=VertMax=60;
          ShowScrollBar(hDemoWin,SB_HORZ,TRUE);
          SetScrollRange(hDemoWin,SB_HORZ,HorzMin,HorzMax,FALSE);
          SetScrollPos(hDemoWin,SB_HORZ,LineHdr.HorzAngle,TRUE);
          ShowScrollBar(hDemoWin,SB_VERT,TRUE);
          SetScrollRange(hDemoWin,SB_VERT,VertMin,VertMax,FALSE);
          SetScrollPos(hDemoWin,SB_VERT,-LineHdr.VertAngle,TRUE);

          //*********** get client window size ************
          GetClientRect(hDemoWin,&ClientRect);        // get the window client area 
          ClientWidth=ClientRect.right-ClientRect.left;
          ClientHeight=ClientRect.bottom-ClientRect.top;

          // set the window size, center the chart 
          ChartWidth=LineHdr.x+(LineHdr.width*PixPerInchX*10)/254+LineHdr.x+WinWidth-ClientWidth;
          ChartHeight=LineHdr.y+(LineHdr.height*PixPerInchY*10)/254+LineHdr.y+WinHeight-ClientHeight;
          //** center the window *
          WinRect.left=(ScrRect.right-ChartWidth)/2;
          WinRect.top=(ScrRect.bottom-ChartHeight)/2;
          MoveWindow(hDemoWin,WinRect.left,WinRect.top,ChartWidth,ChartHeight,TRUE);
          ValidateRect(hDemoWin,NULL);  // No need to repaint 

          ChartType=type;
          result=ShowLine(&LineHdr);
          break;

       case TYPE_HILO:     // show the High-low bar type chart 
          
          //************* set scroll bar *********
          HorzMin=VertMin=-60;                 // allows -60 to +60 view angles 
          HorzMax=VertMax=60;
          ShowScrollBar(hDemoWin,SB_HORZ,TRUE);
          SetScrollRange(hDemoWin,SB_HORZ,HorzMin,HorzMax,FALSE);
          SetScrollPos(hDemoWin,SB_HORZ,HlbHdr.HorzAngle,TRUE);
          ShowScrollBar(hDemoWin,SB_VERT,TRUE);
          SetScrollRange(hDemoWin,SB_VERT,VertMin,VertMax,FALSE);
          SetScrollPos(hDemoWin,SB_VERT,-HlbHdr.VertAngle,TRUE);

          //*********** get client window size ************
          GetClientRect(hDemoWin,&ClientRect);        // get the window client area 
          ClientWidth=ClientRect.right-ClientRect.left;
          ClientHeight=ClientRect.bottom-ClientRect.top;

          // set the window size, center the chart 
          ChartWidth=HlbHdr.x+(HlbHdr.width*PixPerInchX*10)/254+HlbHdr.x+WinWidth-ClientWidth;
          ChartHeight=HlbHdr.y+(HlbHdr.height*PixPerInchY*10)/254+HlbHdr.y+WinHeight-ClientHeight;
          //** center the window *
          WinRect.left=(ScrRect.right-ChartWidth)/2;
          WinRect.top=(ScrRect.bottom-ChartHeight)/2;
          MoveWindow(hDemoWin,WinRect.left,WinRect.top,ChartWidth,ChartHeight,TRUE);
          ValidateRect(hDemoWin,NULL);  // No need to repaint 

          ChartType=type;
          result=ShowHlb(&HlbHdr);
          break;
       
       case TYPE_POINT:     // show the line type chart 
          
          //************* set scroll bar *********
          HorzMin=VertMin=-60;                 // allows -60 to +60 view angles 
          HorzMax=VertMax=60;
          ShowScrollBar(hDemoWin,SB_HORZ,TRUE);
          SetScrollRange(hDemoWin,SB_HORZ,HorzMin,HorzMax,FALSE);
          SetScrollPos(hDemoWin,SB_HORZ,PointHdr.HorzAngle,TRUE);
          ShowScrollBar(hDemoWin,SB_VERT,TRUE);
          SetScrollRange(hDemoWin,SB_VERT,VertMin,VertMax,FALSE);
          SetScrollPos(hDemoWin,SB_VERT,-PointHdr.VertAngle,TRUE);

          //*********** get client window size ************
          GetClientRect(hDemoWin,&ClientRect);        // get the window client area 
          ClientWidth=ClientRect.right-ClientRect.left;
          ClientHeight=ClientRect.bottom-ClientRect.top;

          // set the window size, center the chart 
          ChartWidth=PointHdr.x+(PointHdr.width*PixPerInchX*10)/254+PointHdr.x+WinWidth-ClientWidth;
          ChartHeight=PointHdr.y+(PointHdr.height*PixPerInchY*10)/254+PointHdr.y+WinHeight-ClientHeight;
          //** center the window *
          WinRect.left=(ScrRect.right-ChartWidth)/2;
          WinRect.top=(ScrRect.bottom-ChartHeight)/2;
          MoveWindow(hDemoWin,WinRect.left,WinRect.top,ChartWidth,ChartHeight,TRUE);
          ValidateRect(hDemoWin,NULL);  // No need to repaint 

          ChartType=type;
          result=ShowPoint(&PointHdr);
          break;

       default:
       ;
    }

    SetCursor(hSaveCursor);             // show regular cursor 

    if (result) return FALSE;
    else        return TRUE;
}

/****************************************************************************
    Process Main Window messages
****************************************************************************/
long WINAPI _export DemoWndProc(HWND hWnd,UINT message,WPARAM wParam,LPARAM lParam)
{
    PAINTSTRUCT PaintData;
    MSG msg;
    int x,y,CurObj;

    switch (message) {
        case WM_COMMAND:
            if (TimerSet) SetChartTimer();       // stop the timer

            switch (COMMAND_ID(wParam,lParam)) {
                case IDM_SELECT_BAR:
                    InitBarData();
                    ShowChart(TYPE_BAR);
                    break;

                case IDM_SELECT_PIE:
                    ShowChart(TYPE_PIE);
                    break;

                case IDM_SELECT_LINE:
                    ShowChart(TYPE_LINE);
                    break;

                case IDM_SELECT_HILO:
                    ShowChart(TYPE_HILO);
                    break;

                case IDM_SELECT_POINT:
                    ShowChart(TYPE_POINT);
                    break;

                case IDM_EDIT:             // edit the current chart 
                    if      (ChartType==TYPE_BAR)  EditBar(&BarHdr);
                    else if (ChartType==TYPE_PIE)  EditPie(&PieHdr);
                    else if (ChartType==TYPE_LINE) EditLine(&LineHdr);
                    else if (ChartType==TYPE_HILO) EditHlb(&HlbHdr);
                    else if (ChartType==TYPE_POINT) EditPoint(&PointHdr);
                    
                    ShowChart(ChartType);  // repaint screen 
                    break;

                case IDM_SAVE_AS:             // edit the current chart 
                    if      (ChartType==TYPE_BAR)  SaveBarToFile(&BarHdr,NULL);
                    else if (ChartType==TYPE_PIE)  SavePieToFile(&PieHdr,NULL);
                    else if (ChartType==TYPE_LINE) SaveLineToFile(&LineHdr,NULL);
                    else if (ChartType==TYPE_HILO) SaveHlbToFile(&HlbHdr,NULL);
                    else if (ChartType==TYPE_POINT) SavePointToFile(&PointHdr,NULL);
                    
                    break;

                case IDM_TIMER:
                   SetChartTimer();  // set a timer to show different charts
                   break;
                
                case IDM_EXIT:
                   PostQuitMessage(0);
                   break;

                default:
                break;
            }
            break;

        case WM_TIMER:
            ShowNextChart();       // show next chart on the timer
            break;

        case WM_PAINT:
            BeginPaint(hWnd,&PaintData); 
            EndPaint(hWnd,&PaintData); 
            if      (ChartType==TYPE_BAR)   ShowBar(&BarHdr);
            else if (ChartType==TYPE_PIE)   ShowPie(&PieHdr);
            else if (ChartType==TYPE_LINE)  ShowLine(&LineHdr);
            else if (ChartType==TYPE_HILO)  ShowHlb(&HlbHdr);
            else if (ChartType==TYPE_POINT) ShowPoint(&PointHdr);

            break;

        case WM_VSCROLL:                       // rotate chart vertically 
            switch (wParam) {
                case SB_LINEDOWN:              
                   ShiftView('V',-2,'R');break;// lower vertical view angle by 2 degree 
                case SB_PAGEDOWN:              
                   ShiftView('V',-10,'R');break;// lower vertical view angle by 10 degree 
                case SB_LINEUP:                
                   ShiftView('V',2,'R');break; // increment vertical view angle by 2 degree 
                case SB_PAGEUP:                // previous page 
                   ShiftView('V',10,'R');break;// increment vertical view angle by 10 degree 
                case SB_THUMBTRACK:            // absolute line position as the thump moves
                   ShiftView('V',-(int)(THUMB_POS(wParam,lParam)),'A');break; // show a specified vertical view 
                default:
                   break;
            }
            break;

        case WM_HSCROLL:                        // horizontal scroll bar commands 
            switch (wParam) {
                case SB_LINEDOWN:               
                   ShiftView('H',2,'R');break;  // increment horizontal view angle by 2 degree 
                case SB_PAGEDOWN:               
                   ShiftView('H',10,'R');break; // increment horizontal view angle by 10 degree 
                case SB_LINEUP:                 
                   ShiftView('H',-2,'R');break; // decrement horizontal view angle by 2 degree 
                case SB_PAGEUP:                 // previous page 
                   ShiftView('H',-10,'R');break;// decrement horizontal view angle by 10 degree 
                case SB_THUMBTRACK:             // absolute line position as the thump moves
                   ShiftView('H',(int)(THUMB_POS(wParam,lParam)),'A');break; // show a specified horizontal view 
                default:
                   break;
            }
            break;
        
        case WM_KEYDOWN:                       // rotate chart by using 
            switch (wParam) {                  //  the arrow keys       
                case VK_LEFT:              
                   ShiftView('H',-2,'R');break; 
                case VK_RIGHT:              
                   ShiftView('H',2,'R');break; 
                case VK_UP:              
                   ShiftView('V',2,'R');break; 
                case VK_DOWN:              
                   ShiftView('V',-2,'R');break; 
                case VK_F5:              // draw to printer device context
                   PrintChart();break; 
                case VK_F6:              // draw to metafile to screen
                   PrintMetafile(TRUE);break; 
                case VK_F7:              // draw to metafile to printer
                   PrintMetafile(FALSE);break; 
                case VK_F8:              // draw to metafile to printer
                   PrintBitmap();break;  // print bitmap to screen
                default:
                   break;
            }
            //***** discard remaining keydown messages ****
            while (PeekMessage(&msg,hWnd,WM_KEYDOWN,WM_KEYDOWN,PM_REMOVE|PM_NOYIELD));

            break;

        case WM_LBUTTONDOWN:                // highlight the chart object clicked by mouse
            x=(int)(short)LOWORD(lParam);               // mouse location
            y=(int)(short)HIWORD(lParam);
            if (ChartType==TYPE_BAR) {
               if ((CurObj=LocateBar(&BarHdr,x-BarHdr.x,y-BarHdr.y))>=0) {
                  if (SelectedBar>=0) {      // reset the previously highlighted bar
                     bar[SelectedBar].FrontBrush.lbStyle=BS_SOLID;
                     bar[SelectedBar].SideBrush.lbStyle=BS_SOLID;
                  }
                  SelectedBar=CurObj;        // hilight the selected bar
                  bar[SelectedBar].FrontBrush.lbStyle=BS_HATCHED;
                  bar[SelectedBar].SideBrush.lbStyle=BS_HATCHED;
                  ShowBar(&BarHdr);         // show the modified bar
               }
            }
            else if (ChartType==TYPE_PIE) {
               if ((CurObj=LocatePie(&PieHdr,x-PieHdr.x,y-PieHdr.y))>=0) {
                  if (SelectedPie>=0) {      // reset the previously highlighted bar
                     pie[SelectedPie].FrontBrush.lbStyle=BS_SOLID;
                     pie[SelectedPie].SideBrush.lbStyle=BS_SOLID;
                  }
                  SelectedPie=CurObj;        // hilight the selected bar
                  pie[SelectedPie].FrontBrush.lbStyle=BS_HATCHED;
                  pie[SelectedPie].SideBrush.lbStyle=BS_HATCHED;
                  ShowPie(&PieHdr);         // show the modified bar
               }
            }
            else if (ChartType==TYPE_LINE) {
               if ((CurObj=LocateLine(&LineHdr,x-LineHdr.x,y-LineHdr.y))>=0) {
                  if (SelectedLine>=0) {      // reset the previously highlighted bar
                     line[SelectedLine].FrontBrush.lbStyle=BS_SOLID;
                     line[SelectedLine].SideBrush.lbStyle=BS_SOLID;
                  }
                  SelectedLine=CurObj;        // hilight the selected bar
                  line[SelectedLine].FrontBrush.lbStyle=BS_HATCHED;
                  line[SelectedLine].SideBrush.lbStyle=BS_HATCHED;
                  ShowLine(&LineHdr);         // show the modified bar
               }
            }
            else if (ChartType==TYPE_HILO) {
               if ((CurObj=LocateHlb(&HlbHdr,x-HlbHdr.x,y-HlbHdr.y))>=0) {
                  if (SelectedHlb>=0) {      // reset the previously highlighted hlb
                     hlb[SelectedHlb].FrontBrush.lbStyle=BS_SOLID;
                     hlb[SelectedHlb].SideBrush.lbStyle=BS_SOLID;
                  }
                  SelectedHlb=CurObj;        // hilight the selected hlb
                  hlb[SelectedHlb].FrontBrush.lbStyle=BS_HATCHED;
                  hlb[SelectedHlb].SideBrush.lbStyle=BS_HATCHED;
                  ShowHlb(&HlbHdr);         // show the modified hlb
               }
            }
            else if (ChartType==TYPE_POINT) {
               if ((CurObj=LocatePoint(&PointHdr,x-PointHdr.x,y-PointHdr.y,1))>=0) {
                  if (SelectedPoint>=0) {      // reset the previously highlighted point
                     point[SelectedPoint].SymBrush.lbColor=0x0;
                  }
                  SelectedPoint=CurObj;        // hilight the selected point
                  point[SelectedPoint].SymBrush.lbColor=0x0000FF;  // highlight color
                  ShowPoint(&PointHdr);         // show the modified point
               }
            }
            break;

        case WM_DESTROY:
            //******** delete demo program resources ******
            if (hDevMode) {                 // release printer resources 
               GlobalUnlock(hDevMode);
               GlobalFree(hDevMode);
            }
            if (hPrtDC) DeleteDC(hPrtDC);

            ReleaseDC(hDemoWin,hDC);        // release the demo window handle 

            PostQuitMessage(0);
            break;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
    }
    return (LRESULT)(NULL);
}

/*****************************************************************************
    ShiftView:
    Shift Vertical or Horizontal View Angle.
******************************************************************************/
ShiftView(char HorzVert,int degree,char AbsRel)
{
    switch (ChartType) {
       case TYPE_BAR:
          if (HorzVert=='H') {      // horizontal view shift 
             if (AbsRel=='A') BarHdr.HorzAngle=degree;
             else             BarHdr.HorzAngle+=degree;
             if (BarHdr.HorzAngle>HorzMax) BarHdr.HorzAngle=HorzMax;
             if (BarHdr.HorzAngle<HorzMin) BarHdr.HorzAngle=HorzMin;
             SetScrollPos(hDemoWin,SB_HORZ,BarHdr.HorzAngle,TRUE);
          }
          else {                    // vertical view shift 
             if (AbsRel=='A') BarHdr.VertAngle=degree;
             else             BarHdr.VertAngle+=degree;
             if (BarHdr.VertAngle>VertMax) BarHdr.VertAngle=VertMax;
             if (BarHdr.VertAngle<VertMin) BarHdr.VertAngle=VertMin;
             SetScrollPos(hDemoWin,SB_VERT,-BarHdr.VertAngle,TRUE);
          }
          ShowBar(&BarHdr);         // show the bar chart 
          break;

       case TYPE_PIE:
          if (HorzVert=='H') {      // horizontal view shift 
             if (AbsRel=='A') PieHdr.InitAngle=degree;
             else             PieHdr.InitAngle+=degree;
             if (PieHdr.InitAngle>HorzMax) PieHdr.InitAngle=HorzMax;
             if (PieHdr.InitAngle<HorzMin) PieHdr.InitAngle=HorzMin;
             SetScrollPos(hDemoWin,SB_HORZ,PieHdr.InitAngle,TRUE);
          }
          else {                    // vertical view shift 
             if (AbsRel=='A') PieHdr.VertAngle=degree;
             else             PieHdr.VertAngle-=degree;
             if (PieHdr.VertAngle>VertMax) PieHdr.VertAngle=VertMax;
             if (PieHdr.VertAngle<VertMin) PieHdr.VertAngle=VertMin;
             SetScrollPos(hDemoWin,SB_VERT,PieHdr.VertAngle,TRUE);
          }
          ShowPie(&PieHdr);         // show the pie chart 
          break;

       case TYPE_LINE:
          if (HorzVert=='H') {      // horizontal view shift 
             if (AbsRel=='A') LineHdr.HorzAngle=degree;
             else             LineHdr.HorzAngle+=degree;
             if (LineHdr.HorzAngle>HorzMax) LineHdr.HorzAngle=HorzMax;
             if (LineHdr.HorzAngle<HorzMin) LineHdr.HorzAngle=HorzMin;
             SetScrollPos(hDemoWin,SB_HORZ,LineHdr.HorzAngle,TRUE);
          }
          else {                    // vertical view shift 
             if (AbsRel=='A') LineHdr.VertAngle=degree;
             else             LineHdr.VertAngle+=degree;
             if (LineHdr.VertAngle>VertMax) LineHdr.VertAngle=VertMax;
             if (LineHdr.VertAngle<VertMin) LineHdr.VertAngle=VertMin;
             SetScrollPos(hDemoWin,SB_VERT,-LineHdr.VertAngle,TRUE);
          }
          ShowLine(&LineHdr);         // show the line chart 
          break;

       case TYPE_HILO:
          if (HorzVert=='H') {      // horizontal view shift 
             if (AbsRel=='A') HlbHdr.HorzAngle=degree;
             else             HlbHdr.HorzAngle+=degree;
             if (HlbHdr.HorzAngle>HorzMax) HlbHdr.HorzAngle=HorzMax;
             if (HlbHdr.HorzAngle<HorzMin) HlbHdr.HorzAngle=HorzMin;
             SetScrollPos(hDemoWin,SB_HORZ,HlbHdr.HorzAngle,TRUE);
          }
          else {                    // vertical view shift 
             if (AbsRel=='A') HlbHdr.VertAngle=degree;
             else             HlbHdr.VertAngle+=degree;
             if (HlbHdr.VertAngle>VertMax) HlbHdr.VertAngle=VertMax;
             if (HlbHdr.VertAngle<VertMin) HlbHdr.VertAngle=VertMin;
             SetScrollPos(hDemoWin,SB_VERT,-HlbHdr.VertAngle,TRUE);
          }
          ShowHlb(&HlbHdr);         // show the high-low bar chart 
          break;

       case TYPE_POINT:
          if (HorzVert=='H') {      // horizontal view shift 
             if (AbsRel=='A') PointHdr.HorzAngle=degree;
             else             PointHdr.HorzAngle+=degree;
             if (PointHdr.HorzAngle>HorzMax) PointHdr.HorzAngle=HorzMax;
             if (PointHdr.HorzAngle<HorzMin) PointHdr.HorzAngle=HorzMin;
             SetScrollPos(hDemoWin,SB_HORZ,PointHdr.HorzAngle,TRUE);
          }
          else {                    // vertical view shift 
             if (AbsRel=='A') PointHdr.VertAngle=degree;
             else             PointHdr.VertAngle+=degree;
             if (PointHdr.VertAngle>VertMax) PointHdr.VertAngle=VertMax;
             if (PointHdr.VertAngle<VertMin) PointHdr.VertAngle=VertMin;
             SetScrollPos(hDemoWin,SB_VERT,-PointHdr.VertAngle,TRUE);
          }
          ShowPoint(&PointHdr);         // show the line chart 
          break;

       default:
       ;
    }

    return TRUE;
}

/******************************************************************************
    SetChartTimer:
    Set the timer to show diffrent charts.
*******************************************************************************/
SetChartTimer()
{
    if (TimerSet) KillTimer(hDemoWin,1);
    else          SetTimer(hDemoWin,1,2000,NULL);

    TimerSet=!TimerSet;
 
    ModifyMenu(GetMenu(hDemoWin),IDM_TIMER,MF_BYCOMMAND,IDM_TIMER,TimerSet?"Stop Timer":"Start Timer");

    return TRUE;
} 

/******************************************************************************
    ShowNextChart:
    show the current chart on the timer
*******************************************************************************/
ShowNextChart()
{
    struct StrBarAxis BarAxis;
    struct StrLineAxis LineAxis;
    int i;

    switch (CurChart) {
       case SHOW_BAR_1:         // default bar chart
          InitBarData();
          ChartType=TYPE_BAR;
          break;

       case SHOW_BAR_2:         // one-dimensional
          InitBarData();
          ChartType=TYPE_BAR;

          BarHdr.HorzAngle=BarHdr.VertAngle=0;
          for (i=0;i<BarHdr.BarCount;i++) {   // reduce bar width
             BarHdr.bar[i].width=8;
          } 
          break;

       case SHOW_BAR_3:         // 3d-cylindrical
          InitBarData();
          ChartType=TYPE_BAR;

          BarHdr.BarShape=SHAPE_CYND;
          break;

       case SHOW_BAR_4:         // 3d, axis swap
          InitBarData();
          ChartType=TYPE_BAR;

          //BarHdr.PosFlag=BAR_VERT;               // bar on vertical access with vertical labels
          BarHdr.PosFlag=BAR_VERT_LABEL_HORZ;    // bar on vertical access with horizontal labels
          FarMove(&BarHdr.ax,&BarAxis,sizeof(BarAxis));
          FarMove(&BarHdr.ay,&BarHdr.ax,sizeof(BarAxis));
          FarMove(&BarAxis,&BarHdr.ay,sizeof(BarAxis));
          
          BarHdr.LeftMargin=35;

          for (i=0;i<BarHdr.BarCount;i++) {   // reduce bar width/height
             BarHdr.bar[i].width=6;
             BarHdr.bar[i].depth=5;
             BarHdr.bar[i].ValuePos=VPOS_BOT;
          } 
          strcpy(BarHdr.bar[0].desc,"long label test");
          BarHdr.bar[0].DescBox=false;
          //KillTimer(hDemoWin,1);
          break;

       case SHOW_BAR_5:         // all bars next to each other
          InitBarData();
          ChartType=TYPE_BAR;

          BarHdr.ay.max=50;
          for (i=0;i<BarHdr.BarCount;i++) {
             BarHdr.bar[i].width=BarHdr.bar[i].depth=4;
             if (BarHdr.bar[i].flag!=POS_ABS) BarHdr.bar[i].flag=POS_NEXT;
          }
          break;

       case SHOW_BAR_6:         // all bars stacked
          InitBarData();
          ChartType=TYPE_BAR;

          BarHdr.ay.max=120;
          for (i=0;i<BarHdr.BarCount;i++) {
             if (BarHdr.bar[i].flag!=POS_ABS) BarHdr.bar[i].flag=POS_STACK;
          }
          break;

       case SHOW_BAR_7:         // all bars behind the previous one
          InitBarData();
          ChartType=TYPE_BAR;

          BarHdr.ay.max=50;
          for (i=0;i<BarHdr.BarCount;i++) {
             BarHdr.bar[i].width=BarHdr.bar[i].depth=6;
             if (BarHdr.bar[i].flag!=POS_ABS) BarHdr.bar[i].flag=POS_BEHIND;
          }
          break;

       case SHOW_BAR_8:         // all bars next to each other, 1d
          InitBarData();
          ChartType=TYPE_BAR;
          
          BarHdr.HorzAngle=BarHdr.VertAngle=0;

          BarHdr.ay.max=50;
          for (i=0;i<BarHdr.BarCount;i++) {
             BarHdr.bar[i].width=BarHdr.bar[i].depth=4;
             if (BarHdr.bar[i].flag!=POS_ABS) BarHdr.bar[i].flag=POS_NEXT;
          }
          break;

       case SHOW_PIE_1:         // default pie chart
          InitPieData();
          ChartType=TYPE_PIE;
          break;

       case SHOW_PIE_2:         // pie - one dimensional
          InitPieData();
          ChartType=TYPE_PIE;
          PieHdr.VertAngle=0;
          break;

       case SHOW_PIE_3:         // pie - no offset
          InitPieData();
          ChartType=TYPE_PIE;
          PieHdr.pie[3].flag=POS_CENTER;
          PieHdr.pie[3].solid=TRUE;
          PieHdr.pie[3].DescBox=FALSE;
          break;

       case SHOW_PIE_4:         // pie - no offset, no 3d
          InitPieData();
          ChartType=TYPE_PIE;
          PieHdr.pie[3].flag=POS_CENTER;
          PieHdr.pie[3].solid=TRUE;
          PieHdr.pie[3].DescBox=FALSE;
          PieHdr.VertAngle=0;
          break;

       case SHOW_PIE_5:         // pie - all out, no 3d
          InitPieData();
          ChartType=TYPE_PIE;
          PieHdr.pie[3].solid=TRUE;
          PieHdr.pie[3].DescBox=FALSE;
          PieHdr.VertAngle=0;
          for (i=0;i<PieHdr.PieCount;i++) {
             PieHdr.pie[i].flag=POS_OUT;
             PieHdr.pie[i].pos=5;                // 5 mm pulled out
          } 
          break;

       case SHOW_PIE_6:         // pie - all out, 3d
          InitPieData();
          ChartType=TYPE_PIE;
          PieHdr.pie[3].solid=TRUE;
          PieHdr.pie[3].DescBox=FALSE;
          PieHdr.InitAngle=90;
          for (i=0;i<PieHdr.PieCount;i++) {
             PieHdr.pie[i].flag=POS_OUT;
             PieHdr.pie[i].pos=5;                // 5 mm pulled out
          } 
          break;

       case SHOW_LINE_1:        // default line chart
          InitLineData();
          ChartType=TYPE_LINE;
          break;

       case SHOW_LINE_2:        // default line chart, 1d
          InitLineData();

          LineHdr.HorzAngle=LineHdr.VertAngle=0;
          ChartType=TYPE_LINE;
          break;

       case SHOW_LINE_3:        // default line chart, 1d, no area
          InitLineData();

          LineHdr.HorzAngle=LineHdr.VertAngle=0;
          LineHdr.area=FALSE;
          ChartType=TYPE_LINE;
          break;

       case SHOW_LINE_4:        // default line chart, 3d, no area
          InitLineData();

          LineHdr.area=FALSE;
          ChartType=TYPE_LINE;
          break;

       case SHOW_LINE_5:        // default line chart, 1d, no area, axis swap
          InitLineData();

          LineHdr.HorzAngle=LineHdr.VertAngle=0;
          LineHdr.area=FALSE;
          LineHdr.line[0].desc[0]=0;

          LineHdr.PosFlag=BAR_VERT;
          FarMove(&LineHdr.ax,&LineAxis,sizeof(LineAxis));
          FarMove(&LineHdr.ay,&LineHdr.ax,sizeof(LineAxis));
          FarMove(&LineAxis,&LineHdr.ay,sizeof(LineAxis));

          ChartType=TYPE_LINE;
          break;

       case SHOW_HILO_1:        // default hilo chart
          InitHlbData();
          ChartType=TYPE_HILO;
          break;

       case SHOW_HILO_2:        // hilo chart, 1d
          InitHlbData();
          HlbHdr.HorzAngle=HlbHdr.VertAngle=0;
          
          ChartType=TYPE_HILO;
          break;

       case SHOW_HILO_3:        // hilo chart, 1d, stock chart
          InitHlbData();
          HlbHdr.HorzAngle=HlbHdr.VertAngle=0;
          
          HlbHdr.HlbCount=StockHlbCount;
          HlbHdr.hlb=pHlb;
          HlbHdr.ax.Interval1=10;
          HlbHdr.ax.Interval2=20;
          HlbHdr.ax.min=0;
          HlbHdr.ax.max=6*StockHlbCount;

          ChartType=TYPE_HILO;
          break;

       case SHOW_HILO_4:        // hilo chart, 1d, stock chart, one pixel bars
          InitHlbData();
          HlbHdr.HorzAngle=HlbHdr.VertAngle=0;
          
          HlbHdr.HlbCount=StockHlbCount;
          HlbHdr.hlb=pHlb;
          HlbHdr.ax.Interval1=10;
          HlbHdr.ax.Interval2=20;
          HlbHdr.ax.min=0;
          HlbHdr.ax.max=6*StockHlbCount;
          for (i=0;i<StockHlbCount;i++) pHlb[i].width=0;   // one pixel

          ChartType=TYPE_HILO;
          break;

       case SHOW_POINT_1:       // default point
          InitPointData();
          ChartType=TYPE_POINT;
          break;

       case SHOW_POINT_2:       // default point, 1d
          InitPointData();
          ChartType=TYPE_POINT;
          PointHdr.HorzAngle=PointHdr.VertAngle=0;
          break;

       default:
       ;
    }

    ShowChart(ChartType);

    CurChart++;
    if (CurChart==SHOW_TOTAL) CurChart=0;  // restart from the beginning

    return TRUE;
}
 
/******************************************************************************
    PrintChart:
    Print the current chart to the current printer.
*******************************************************************************/
PrintChart()
{
    HCURSOR hSaveCursor;
    int SaveChartType;
    char JobName[32];


    if (!PrinterInitialized) InitPrinter();   // initialize the current printer 

    if (hPrtDC==NULL) return TRUE;   // chart not selected 
    if (ChartType==TYPE_NONE) return TRUE;  

    SaveChartType=ChartType;
    ChartType=TYPE_NONE;              // disable screen painting temporarily      


    hSaveCursor=SetCursor(hWaitCursor);         // show hour glass 
    strcpy(JobName,"Demo Chart Print");
    Escape(hPrtDC,STARTDOC,strlen(JobName),JobName,0L);   // begin print job 
    
    if (SaveChartType==TYPE_BAR) {     // print bar chart 
        BarHdr.hDC=hPrtDC;
        ShowBar(&BarHdr);
    }
    else if (SaveChartType==TYPE_PIE) {// print pie chart 
        PieHdr.hDC=hPrtDC;
        ShowPie(&PieHdr);
    }
    else if (SaveChartType==TYPE_LINE) {// print line chart 
        LineHdr.hDC=hPrtDC;
        ShowLine(&LineHdr);
    }
    else if (SaveChartType==TYPE_HILO) {// print high-low bar chart 
        HlbHdr.hDC=hPrtDC;
        ShowHlb(&HlbHdr);
    }
    else if (SaveChartType==TYPE_POINT) {// print point chart 
        PointHdr.hDC=hPrtDC;
        ShowPoint(&PointHdr);
    }

    Escape(hPrtDC,NEWFRAME,0,0L,0L);
    Escape(hPrtDC,ENDDOC,0,0L,0L);     // end of print job 
    SetCursor(hSaveCursor);             // show regular cursor 

    //********** reset to screen painting ****
    if      (SaveChartType==TYPE_BAR)   BarHdr.hDC=hDC;
    else if (SaveChartType==TYPE_PIE)   PieHdr.hDC=hDC;
    else if (SaveChartType==TYPE_LINE)  LineHdr.hDC=hDC;
    else if (SaveChartType==TYPE_HILO)  HlbHdr.hDC=hDC;
    else if (SaveChartType==TYPE_POINT) PointHdr.hDC=hDC;

    ChartType=SaveChartType;
    
    return TRUE;
}

/******************************************************************************
    PrintMetafile:
    Print the chart to metafile.  Print from the metafile to the window
*******************************************************************************/
PrintMetafile(BOOL ToScreen)
{
    int SaveX,SaveY,SaveId;
    HDC hMetaDC;
    HMETAFILE hMF;
    char JobName[32];


    if (ChartType==TYPE_NONE) return TRUE;  

    if (NULL==(hMetaDC=CreateMetaFile(NULL))) return TRUE;

    if (ChartType==TYPE_BAR) {    // print bar chart 
        SaveX=BarHdr.x;               // print to 0,0
        SaveY=BarHdr.y;
        BarHdr.x=BarHdr.y=0;
        BarHdr.hDC=hMetaDC;
        ShowBar(&BarHdr);

        BarHdr.hDC=hDC;               // restore the structure
        BarHdr.x=SaveX;
        BarHdr.y=SaveY;
    }
    else if (ChartType==TYPE_PIE) {// print pie chart 
        SaveX=PieHdr.x;               // print to 0,0
        SaveY=PieHdr.y;
        PieHdr.x=PieHdr.y=0;
        PieHdr.hDC=hMetaDC;
        ShowPie(&PieHdr);

        PieHdr.hDC=hDC;               // restore the structure
        PieHdr.x=SaveX;
        PieHdr.y=SaveY;
    }
    else if (ChartType==TYPE_LINE) {// print line chart 
        SaveX=LineHdr.x;               // print to 0,0
        SaveY=LineHdr.y;
        LineHdr.x=LineHdr.y=0;
        LineHdr.hDC=hMetaDC;
        ShowLine(&LineHdr);

        LineHdr.hDC=hDC;               // restore the structure
        LineHdr.x=SaveX;
        LineHdr.y=SaveY;
    }
    else if (ChartType==TYPE_HILO) {// print high-low bar chart 
        SaveX=HlbHdr.x;               // print to 0,0
        SaveY=HlbHdr.y;
        HlbHdr.x=HlbHdr.y=0;
        HlbHdr.hDC=hMetaDC;
        ShowHlb(&HlbHdr);

        HlbHdr.hDC=hDC;               // restore the structure
        HlbHdr.x=SaveX;
        HlbHdr.y=SaveY;
    }
    else if (ChartType==TYPE_POINT) {// print point chart 
        SaveX=PointHdr.x;               // print to 0,0
        SaveY=PointHdr.y;
        PointHdr.x=PointHdr.y=0;
        PointHdr.hDC=hMetaDC;
        ShowPoint(&PointHdr);

        PointHdr.hDC=hDC;               // restore the structure
        PointHdr.x=SaveX;
        PointHdr.y=SaveY;
    }

    // play the metafile
    hMF=CloseMetaFile(hMetaDC);


    
    if (ToScreen) {
       // set the units for our window
       SaveId=SaveDC(hDC);                 // save our DC
       SetMapMode(hDC,MM_ANISOTROPIC);     // use customized units
       SetWindowExtEx(hDC,RES_META_DC,RES_META_DC,NULL); // logical dimension
       SetViewportExtEx(hDC,PixPerInchX,PixPerInchY,NULL); // equivalent device dimension
       SetViewportOrgEx(hDC,SaveX,SaveY,NULL);    // set view port origins 

       PlayMetaFile(hDC,hMF);// draw to screen

       RestoreDC(hDC,SaveId);
    }
    else {                              // draw to printer
       DOCINFO doc;
       if (!PrinterInitialized) InitPrinter();               // initialize the current printer 
       strcpy(JobName,"Metafile Chart Print");

       FarMemSet(&doc,0,sizeof(doc));
       doc.cbSize=sizeof(doc);
       doc.lpszDocName=JobName;
       doc.lpszOutput=NULL;
       StartDoc(hPrtDC,&doc);
       StartPage(hPrtDC);

       // set the units for printer
       SaveId=SaveDC(hPrtDC);                 // save our DC
       SetMapMode(hPrtDC,MM_ANISOTROPIC);     // use customized units
       SetWindowExtEx(hPrtDC,RES_META_DC,RES_META_DC,NULL); // logical dimension
       SetViewportExtEx(hPrtDC,GetDeviceCaps(hPrtDC,LOGPIXELSX),GetDeviceCaps(hPrtDC,LOGPIXELSY),NULL); // equivalent device dimension
       SetViewportOrgEx(hPrtDC,SaveX,SaveY,NULL);    // set view port origins 

       PlayMetaFile(hPrtDC,hMF);        // draw to printer

       EndPage(hPrtDC);
       EndDoc(hPrtDC);

       RestoreDC(hPrtDC,SaveId);
    }

    DeleteMetaFile(hMF);

    return TRUE;
}

/******************************************************************************
    PrintBitmap:
    Print a chart from a bitmap to the screen.
*******************************************************************************/
PrintBitmap()
{
    int hChart;
    HBITMAP hBM,hOldBM;
    RECT WinRect;
    BITMAP bm;
    HDC hMemDC;

    hChart=cCreateChart(CHART_BAR,1,1);  // create a sample chart

    cSetDevice(hChart,hDC);              // set the device context
    cReqBitmap(hChart,TRUE,FALSE);       // request bitmap output
    
    cShowChart(hChart);                  // create the bitmap
    hBM=(HBITMAP)cGetBitmap(hChart);     // retrieve the bitmap

    // select the bitmap in a memory device context
    GetObject(hBM,sizeof(BITMAP),(LPBYTE)&bm);     // get the dimension of bit map 
    hMemDC=CreateCompatibleDC(hDC);
    hOldBM=SelectObject(hMemDC,hBM);

    // display the bitmap in the window
    GetWindowRect(hDemoWin,&WinRect);           // get the current window dimensions 
    StretchBlt(hDC,0,0,WinRect.right-WinRect.left,WinRect.bottom-WinRect.top,
               hMemDC,0,0,bm.bmWidth,bm.bmHeight,SRCCOPY);

    // release resources
    SelectObject(hMemDC,hOldBM);
    DeleteObject(hBM);
    DeleteDC(hMemDC);

    return TRUE;
}

/******************************************************************************
    InitPrinter:
    Initialize the current printer variables
*******************************************************************************/
InitPrinter()
{
    int bytes;
    char driver[60];

    hPrtDC=0;

    if (!CurrentPrinter(PrinterName,PrinterDriver,PrinterPort)) return FALSE; // get current printer info 

    //************** create a printer device context ********
    lstrcpy(driver,PrinterDriver);
    driver[lstrlen(driver)-4]=0;            // strip the .drv extension 
    if (NULL==(hPrtDC=CreateDC(driver,PrinterName,PrinterPort,NULL))) {
        MessageBeep(0);
        return FALSE;
    }
    
    PrinterInitialized=TRUE;

    return TRUE;
}

/******************************************************************************
    CurrentPrinter:
    Get information (name,driver,port) for the currently selected printer.
    The function returns FALSE if the current printer not set.
******************************************************************************/
BOOL CurrentPrinter(LPSTR name,LPSTR driver,LPSTR port)
{
    char PrintData[80],ext[5];
    LPSTR temp,name1,driver1=0,port1=0;
    int len;

    name[0]=driver[0]=port[0]=0;       // initialized the output variables 

    if (!GetProfileString("windows","device",(LPSTR) "",PrintData,80)) return FALSE;

    temp=name1=PrintData;              // beginning pointer for the scan 

    while(*temp) {                     // scan until a NULL is found 
       if (*temp==',') {               // a delimited found 
          *temp=0;                     // NULL terminate the previous string, and advance the pointer 
          temp++;
          while (*temp==' ') temp=AnsiNext(temp);   // skip leading spaces 
          if (!driver1) driver1=temp;  // Initiate the driver string if not found yet 
          else {                       // else it is the last string, i.e. the port string 
             port1=temp;
             break;                    // all strings parsed 
          }
       }
       else temp=AnsiNext(temp);       // advance the pointer 
    }

    lstrcpy(name,name1);               // copy the strings to the user specified area 
    lstrcpy(port,port1);
    lstrcpy(driver,driver1);
    
    // add .drv extension to the driver name, if necessary 
    len=lstrlen(driver);
    if (len<=4) lstrcat(driver,".drv");
    else {
       lstrcpy(ext,&driver[len-4]);
       strupr(ext);
       if (lstrcmp(ext,".DRV")!=0) lstrcat(driver,".DRV");
    }
    
    return TRUE;
}

/******************************************************************************
    Support Routines.
******************************************************************************/

/*****************************************************************************
    StringTrim:
    Remove spaces from the left and right of a NULL terminated string.
******************************************************************************/
void StringTrim(LPSTR string)  
{
    RightTrim(string);LeftTrim(string);
}

/******************************************************************************
    RightTrim:
    Remove spaces on the right of a string.
******************************************************************************/
void RightTrim(LPSTR string)  
{
    int i,TempLen;

    TempLen=lstrlen(string);
    for (i=TempLen-1;i>=0 && string[i] == ' ';i--);
    string[i+1] = '\0';
}

/******************************************************************************
    LeftTrim:
    Trim initial spaces from the string.
*******************************************************************************/
void LeftTrim(LPSTR string)    
{
    int i,TempLen,BeginPoint;char TempStr[300];

    TempLen=lstrlen(string);
    for (i=0;i < TempLen && string[i] == ' ';i++);
    BeginPoint=i;
    for (i=BeginPoint;i<TempLen;i++) TempStr[i-BeginPoint]=string[i];
    TempStr[TempLen-BeginPoint] = '\0';
    lstrcpy(string,TempStr);
}

/******************************************************************************
     GetNumVal:
     Extract a numeric value from the dialog item.  Display error if a non-
     numeric value is encountered and return a false result.
*******************************************************************************/
BOOL GetNumVal(HWND hWin,int ControlId,int *pInt)
{
    BOOL ErrorFlag;

    *pInt=GetDlgItemInt(hWin,ControlId,&ErrorFlag,(BOOL) 0);
    if (!ErrorFlag) {
       MessageBox(hWin,"Must Enter a Valid Number",NULL,MB_OK);
       SetFocus(GetDlgItem(hWin,ControlId));
       return (FALSE);
    }
    return TRUE;
}

/******************************************************************************
    OurPrintf:
    This routine formats and display a given set of arguments.  The format
    is given by the first argument.  The argument 2 to n contain the
    arguments for the specified format.  The function uses MessageBox to
    display the formatted string.
******************************************************************************/
int OurPrintf(LPBYTE fmt,...)
{
    LPSTR ArgStart;
    char string[302];
    HWND hWnd;

    ArgStart=(LPSTR) &fmt;  // pointer to first argument
    ArgStart=&ArgStart[4];  // go past the first argument
    wvsprintf(string,fmt,ArgStart);

    if (NULL!=(hWnd=FindWindow("DBWin",NULL))) { // debug window open
        if (NULL!=(hWnd=GetWindow(hWnd,GW_CHILD))) {    // get the edit window
           int len=lstrlen(string);
           string[len]=0xd;
           string[len+1]=0xa;
           string[len+2]=0;
           #if defined (WIN32)
              SendMessage(hWnd,EM_SETSEL,32767,32767);  // show the last line
              SendMessage(hWnd,EM_REPLACESEL,0,(DWORD)(LPSTR)string);
              SendMessage(hWnd,EM_SCROLLCARET,0,0);     // scroll caret into view
           #else
              SendMessage(hWnd,EM_SETSEL,0,MAKELPARAM(32767,32767));  // show the last line
              SendMessage(hWnd,EM_REPLACESEL,0,(DWORD)(LPSTR)string);
              SendMessage(hWnd,EM_SETSEL,0,MAKELPARAM(32767,32767));  // show the last line
           #endif
        }
    }
    else MessageBox(NULL,string,NULL,MB_OK);
    return TRUE;
}

/******************************************************************************
    FarMemSet:
    This function every byte of the given far buffer to the specified character.
******************************************************************************/
void FarMemSet(void far *ptr,char chr, UINT count)
{
    UINT i;
    LPSTR lptr;

    lptr=(LPSTR)ptr;

    for (i=0;i<count;i++) lptr[i]=chr;
}

/******************************************************************************
    Copy a specified number of bytes from the source to the destination location.
    Both the source and destination locations are specified by using far pointers.
*******************************************************************************/
void FarMove(void far *src,void far *dest,UINT count)
{
    #if defined (WIN32)
       memmove(dest,src,count);
    #else
       WORD SrcSeg,DestSeg,SrcOff,DestOff;

       if (count==0) return;                   // nothing to move

       SrcSeg=(WORD)((DWORD)src>>16);          // segment and offset of source and destination
       SrcOff=(WORD)(((DWORD)src<<16)>>16);
       DestSeg=(WORD)((DWORD)dest>>16);
       DestOff=(WORD)(((DWORD)dest<<16)>>16);

       movedata(SrcSeg,SrcOff,DestSeg,DestOff,count);
    #endif
    return;
}


